@@ -100,15 +100,15 @@ class AssetsAccount : public Account {
 	
 	protected:
 
-		AssetsType at_type;
+		int at_type;
 		double d_initbal;
 		bool b_closed;
-		QString s_maintainer;
+		QString s_maintainer, s_group;
 		Currency *o_currency;
 		
 	public:
 
-		AssetsAccount(Budget *parent_budget, AssetsType initial_type, QString initial_name, double initial_balance = 0.0, QString initial_description = QString::null);
+		AssetsAccount(Budget *parent_budget, int initial_type, QString initial_name, double initial_balance = 0.0, QString initial_description = QString());
 		AssetsAccount(Budget *parent_budget, QXmlStreamReader *xml, bool *valid);
 		AssetsAccount(Budget *parent_budget);
 		AssetsAccount();
@@ -127,9 +127,18 @@ class AssetsAccount : public Account {
 		void setClosed(bool close_account = true);
 		const QString &maintainer() const;
 		void setMaintainer(QString maintainer_name);
-		virtual AccountType type() const;
-		void setAccountType(AssetsType new_type);
-		virtual AssetsType accountType() const;
+		AccountType type() const;
+		void setAccountType(int new_type);
+		int accountType() const;
+		void setAccountTypeName(const QString &new_type, bool localized = false);
+		QString accountTypeName(bool localized = false, bool plural = false) const;
+		QString group() const;
+		void setGroup(QString group_name);
+		bool isCreditCard() const;
+		bool isDebt() const;
+		bool isLiabilities() const;
+		bool isSecurities() const;
+		bool isTypeOther() const;
 		Currency *currency() const;
 		void setCurrency(Currency *new_currency);
 
@@ -140,10 +149,10 @@ template<class type> class AccountList : public EqonomizeList<type> {
 	public:
 		AccountList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), account_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), account_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::insert(qLowerBound(QList<type>::begin(), QList<type>::end(), value, account_list_less_than), value);
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, account_list_less_than), value);
 		}
 };
 
@@ -151,7 +160,7 @@ class CategoryAccount : public Account {
 
 	public:
 
-		CategoryAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString::null);
+		CategoryAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString());
 		CategoryAccount(Budget *parent_budget, QXmlStreamReader *xml, bool *valid);
 		CategoryAccount(Budget *parent_budget);
 		CategoryAccount();
@@ -191,7 +200,7 @@ class IncomesAccount : public CategoryAccount {
 	
 	public:
 
-		IncomesAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString::null);
+		IncomesAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString());
 		IncomesAccount(Budget *parent_budget, QXmlStreamReader *xml, bool *valid);
 		IncomesAccount(Budget *parent_budget);
 		IncomesAccount();
@@ -199,8 +208,6 @@ class IncomesAccount : public CategoryAccount {
 		virtual ~IncomesAccount();
 
 		virtual AccountType type() const;
-		IncomesAccount *parentAccount() const;
-		void setParentAccount(IncomesAccount *account);
 
 };
 
@@ -208,7 +215,7 @@ class ExpensesAccount : public CategoryAccount {
 	
 	public:
 
-		ExpensesAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString::null);
+		ExpensesAccount(Budget *parent_budget, QString initial_name, QString initial_description = QString());
 		ExpensesAccount(Budget *parent_budget, QXmlStreamReader *xml, bool *valid);
 		ExpensesAccount(Budget *parent_budget);
 		ExpensesAccount();
