@@ -30,7 +30,9 @@
 #include <QtCharts/QChart>
 #include <QtCharts/QAbstractSeries>
 #include <QtCharts/QPieSlice>
+#include <QtCharts/QBarSet>
 QT_CHARTS_USE_NAMESPACE
+class QGraphicsItem;
 #else
 class QGraphicsScene;
 class QGraphicsView;
@@ -39,10 +41,12 @@ class QButtonGroup;
 class QCheckBox;
 class QComboBox;
 class QPushButton;
-
+class QRadioButton;
 class QDateEdit;
 
+class AccountsCombo;
 class CategoryAccount;
+class AssetsAccount;
 class Budget;
 
 class CategoriesComparisonChart : public QWidget {
@@ -64,18 +68,22 @@ class CategoriesComparisonChart : public QWidget {
 		QDateEdit *fromEdit, *toEdit;
 		QPushButton *nextYearButton, *prevYearButton, *nextMonthButton, *prevMonthButton;
 		QPushButton *saveButton, *printButton;
+		QRadioButton *percentButton, *valueButton;
 #ifdef QT_CHARTS_LIB
 		QChartView *view;
 		QChart *chart;
 		QAbstractSeries *series;
+		QAbstractAxis *axis_x, *axis_y;
 		QComboBox *themeCombo;
 		QComboBox *typeCombo;
+		QGraphicsItem *point_label;
 #else
 		QGraphicsScene *scene;
 		QGraphicsView *view;
 #endif		
 		QButtonGroup *typeGroup;
 		QComboBox *sourceCombo;
+		AccountsCombo *accountCombo;
 #ifndef QT_CHARTS_LIB
 		void resizeEvent(QResizeEvent*);
 #endif
@@ -97,9 +105,11 @@ class CategoriesComparisonChart : public QWidget {
 		void prevYear();
 		void nextYear();
 		void sourceChanged(int);
+		void valueTypeToggled(int, bool);
 #ifdef QT_CHARTS_LIB
 		void themeChanged(int);
 		void typeChanged(int);
+		void onSeriesHovered(bool, int, QBarSet*);
 		void sliceHovered(QPieSlice*, bool);
 		void sliceClicked(QPieSlice*);
 		void legendClicked();
