@@ -31,7 +31,10 @@ class QRadioButton;
 class QDateEdit;
 class QTextEdit;
 
+class AccountsCombo;
+class DescriptionsCombo;
 class CategoryAccount;
+class AssetsAccount;
 class Budget;
 
 class CategoriesComparisonReport : public QWidget {
@@ -48,9 +51,10 @@ class CategoriesComparisonReport : public QWidget {
 		QString source;
 		QDate from_date, to_date;
 		CategoryAccount *current_account;
-		QString current_description, current_payee;
-		bool has_empty_description, has_empty_payee;
+		QString current_tag;
 		bool b_extra;
+		bool block_display_update;
+		int first_source_account_index;
 		
 		QTextEdit *htmlview;
 		QCheckBox *fromButton;
@@ -58,8 +62,11 @@ class CategoriesComparisonReport : public QWidget {
 		QPushButton *nextYearButton, *prevYearButton, *nextMonthButton, *prevMonthButton;
 		QPushButton *saveButton, *printButton;
 		QCheckBox *valueButton, *dailyButton, *monthlyButton, *yearlyButton, *countButton, *perButton;
-		QComboBox *sourceCombo, *descriptionCombo, *payeeCombo;
+		QComboBox *sourceCombo;
+		DescriptionsCombo *descriptionCombo, *payeeCombo;
+		AccountsCombo *accountCombo;
 		QRadioButton *descriptionButton, *payeeButton, *subsButton;
+		QRadioButton *monthsButton, *yearsButton, *tagsButton, *totalButton;
 		QWidget *payeeDescriptionWidget;
 
 	public slots:
@@ -67,6 +74,7 @@ class CategoriesComparisonReport : public QWidget {
 		void resetOptions();
 		void updateTransactions();
 		void updateAccounts();
+		void updateTags();
 		void updateDisplay();
 		void save();
 		void print();
@@ -78,11 +86,12 @@ class CategoriesComparisonReport : public QWidget {
 		void prevYear();
 		void nextYear();
 		void sourceChanged(int);
-		void descriptionChanged(int);
-		void payeeChanged(int);
+		void descriptionChanged();
+		void payeeChanged();
 		void descriptionToggled(bool);
 		void payeeToggled(bool);
 		void subsToggled(bool);
+		void columnsToggled(int, bool);
 
 };
 
