@@ -37,13 +37,14 @@
 #include <QDateEdit>
 #include <QMessageBox>
 #include <QHeaderView>
+#include <QKeyEvent>
 #include <QRadioButton>
 #include <QCompleter>
 #include <QStandardItemModel>
 #include <QStandardPaths>
 #include <QDirModel>
-#include <QDesktopServices>
 #include <QFileDialog>
+#include <QInputDialog>
 
 #include "accountcombobox.h"
 #include "budget.h"
@@ -59,10 +60,17 @@ extern QString last_associated_file_directory;
 
 extern void setColumnTextWidth(QTreeWidget *w, int i, QString str);
 extern void setColumnDateWidth(QTreeWidget *w, int i);
-extern void setColumnMoneyWidth(QTreeWidget *w, int i, double v = 9999999.99);
-extern void setColumnValueWidth(QTreeWidget *w, int i, double v, int d = -1);
+extern void setColumnMoneyWidth(QTreeWidget *w, int i, Budget *budget, double v = 9999999.99, int d = -1);
 extern void setColumnStrlenWidth(QTreeWidget *w, int i, int l);
 
+EqonomizeRadioButton::EqonomizeRadioButton(const QString &text, QWidget *parent) : QRadioButton(text, parent) {}
+void EqonomizeRadioButton::keyPressEvent(QKeyEvent *event) {
+	QRadioButton::keyPressEvent(event);
+	if(event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) {
+		emit returnPressed();
+	}
+}
+
 class MultiItemListViewItem : public QTreeWidgetItem {
 
 	Q_DECLARE_TR_FUNCTIONS(MultiItemListViewItem)
@@ -85,22 +93,13 @@ class MultiItemListViewItem : public QTreeWidgetItem {
 
 MultiItemListViewItem::MultiItemListViewItem(Transaction *trans, Currency *cur, bool deposit) : QTreeWidgetItem() {
 	setTransaction(trans, cur, deposit);
-	setTextAlignment(2, Qt::AlignRight | Qt::AlignVCenter);
 	setTextAlignment(3, Qt::AlignRight | Qt::AlignVCenter);
 }
 bool MultiItemListViewItem::operator<(const QTreeWidgetItem &i_pre) const {
 	int col = 0;
 	if(treeWidget()) col = treeWidget()->sortColumn();
 	MultiItemListViewItem *i = (MultiItemListViewItem*) &i_pre;
-	if(col == 2) {
-		double value1 = 0.0;
-		double value2 = 0.0;
-		if(b_deposit && o_trans->value() < 0.0) value1 = -o_trans->value();
-		else if(!b_deposit && o_trans->value() >= 0.0) value1 = o_trans->value();
-		if(i->isDeposit() && i->transaction()->value() < 0.0) value2 = -i->transaction()->value();
-		else if(!i->isDeposit() && i->transaction()->value() >= 0.0) value2 = i->transaction()->value();
-		return value1 < value2;
-	} else if(col == 3) {
+	if(col == 3) {
 		double value1 = 0.0;
 		double value2 = 0.0;
 		if(!b_deposit && o_trans->value() < 0.0) value1 = -o_trans->value();
@@ -126,11 +125,8 @@ void MultiItemListViewItem::setTransaction(Transaction *trans, Currency *cur, bo
 	if(deposit) value = trans->toValue();
 	else value = trans->fromValue();
 	setText(1, trans->description());
-	//setText(2, deposit ? trans->fromAccount()->nameWithParent() : trans->toAccount()->nameWithParent());
-	if(deposit) setText(2, value >= 0.0 ? QString::null : cur->formatValue(-value));
-	else setText(2, value < 0.0 ? QString::null : cur->formatValue(value));
-	if(!deposit) setText(3, value >= 0.0 ? QString::null : cur->formatValue(-value));
-	else setText(3, value < 0.0 ? QString::null : cur->formatValue(value));
+	setText(2, deposit ? trans->fromAccount()->nameWithParent() : trans->toAccount()->nameWithParent());
+	setText(3, cur->formatValue(deposit ? value : -value));
 	if(trans->type() == TRANSACTION_TYPE_INCOME) {
 		if(((Income*) trans)->security()) setText(0, tr("Dividend"));
 		else if(value >= 0) setText(0, tr("Income"));
@@ -195,10 +191,12 @@ EditDebtPaymentDialog::EditDebtPaymentDialog(Budget *budg, QWidget *parent, Asse
 	
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
+	connect(editWidget, SIGNAL(addmodify()), this, SLOT(accept()));
+	
+	editWidget->focusFirst();
 	
 }
 EditDebtPaymentDialog::~EditDebtPaymentDialog() {}
@@ -206,6 +204,10 @@ void EditDebtPaymentDialog::accept() {
 	if(!editWidget->validValues()) return;
 	QDialog::accept();
 }
+void EditDebtPaymentDialog::keyPressEvent(QKeyEvent *e) {
+	if(e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) return;
+	QDialog::keyPressEvent(e);
+}
 
 EditMultiAccountDialog::EditMultiAccountDialog(Budget *budg, QWidget *parent, bool create_expenses, bool extra_parameters, bool allow_account_creation) : QDialog(parent, 0) {
 
@@ -219,11 +221,11 @@ EditMultiAccountDialog::EditMultiAccountDialog(Budget *budg, QWidget *parent, bo
 	
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
 	
+	editWidget->focusFirst();
 }
 EditMultiAccountDialog::~EditMultiAccountDialog() {}
 void EditMultiAccountDialog::accept() {
@@ -234,7 +236,10 @@ void EditMultiAccountDialog::reject() {
 	editWidget->reject();
 	QDialog::reject();
 }
-
+void EditMultiAccountDialog::keyPressEvent(QKeyEvent *e) {
+	if(e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) return;
+	QDialog::keyPressEvent(e);
+}
 
 EditMultiItemDialog::EditMultiItemDialog(Budget *budg, QWidget *parent, AssetsAccount *default_account, bool extra_parameters, bool allow_account_creation) : QDialog(parent, 0) {
 	setWindowTitle(tr("Split Transaction"));
@@ -245,11 +250,12 @@ EditMultiItemDialog::EditMultiItemDialog(Budget *budg, QWidget *parent, AssetsAc
 	
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
 	
+	editWidget->focusFirst();
+	
 }
 EditMultiItemDialog::~EditMultiItemDialog() {}
 void EditMultiItemDialog::accept() {
@@ -260,6 +266,10 @@ void EditMultiItemDialog::reject() {
 	editWidget->reject();
 	QDialog::reject();
 }
+void EditMultiItemDialog::keyPressEvent(QKeyEvent *e) {
+	if(e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) return;
+	QDialog::keyPressEvent(e);
+}
 
 EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAccount *default_account, bool extra_parameters, bool allow_account_creation) : QWidget(parent), budget(budg), b_extra(extra_parameters), b_create_accounts(allow_account_creation) {
 
@@ -294,7 +304,7 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 	((QStandardItemModel*) descriptionEdit->completer()->model())->sort(1);
 
 	grid->addWidget(new QLabel(tr("Date:")), 1, 0);
-	dateEdit = new QDateEdit(QDate::currentDate());
+	dateEdit = new EqonomizeDateEdit(this);
 	dateEdit->setCalendarPopup(true);
 	grid->addWidget(dateEdit, 1, 1);
 
@@ -344,7 +354,13 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 		payeeEdit = NULL;
 	}
 	
-	grid->addWidget(new QLabel(tr("Associated file:"), this), b_extra ? 4 : 3, 0);
+	grid->addWidget(new QLabel(tr("Tags:")), b_extra ? 4 : 3, 0);
+	tagButton = new TagButton(false, allow_account_creation, budget, this);
+	grid->addWidget(tagButton, b_extra ? 4 : 3, 1);
+	tagButton->updateTags();
+	connect(tagButton, SIGNAL(newTagRequested()), this, SLOT(newTag()));
+	
+	grid->addWidget(new QLabel(tr("Associated file:"), this), b_extra ? 5 : 4, 0);
 	QHBoxLayout *fileLayout = new QHBoxLayout();
 	fileEdit = new QLineEdit(this);
 	QCompleter *completer = new QCompleter(this);
@@ -360,12 +376,16 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 	openFileButton->setAutoDefault(false);
 	fileLayout->addWidget(openFileButton);
 	openFileButton->setFocusPolicy(Qt::ClickFocus);
-	grid->addLayout(fileLayout, b_extra ? 4 : 3, 1);
+	grid->addLayout(fileLayout, b_extra ? 5 : 4, 1);
+	
+	grid->addWidget(new QLabel(tr("Comments:")), b_extra ? 6 : 5, 0);
+	commentEdit = new QLineEdit();
+	grid->addWidget(commentEdit, b_extra ? 6 : 5, 1);
 
 	box1->addWidget(new QLabel(tr("Transactions:")));
 	QHBoxLayout *box2 = new QHBoxLayout();
 	box1->addLayout(box2);
-	transactionsView = new EqonomizeTreeWidget();
+	transactionsView = new EqonomizeTreeWidget(this);
 	transactionsView->setSortingEnabled(true);
 	transactionsView->sortByColumn(0, Qt::AscendingOrder);
 	transactionsView->setAllColumnsShowFocus(true);
@@ -373,19 +393,26 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 	QStringList headers;
 	headers << tr("Type");
 	headers << tr("Description", "Transaction description property (transaction title/generic article name)");
-	//headers << tr("Account/Category");
-	headers << tr("Payment");
-	headers << tr("Deposit");
+	headers << tr("Account/Category");
+	headers << tr("Value");
 	transactionsView->setHeaderLabels(headers);
 	transactionsView->setRootIsDecorated(false);
-	setColumnStrlenWidth(transactionsView, 0, 15);
-	setColumnStrlenWidth(transactionsView, 1, 25);
-	setColumnMoneyWidth(transactionsView, 2);
-	setColumnMoneyWidth(transactionsView, 3);
+	QFontMetrics fm(transactionsView->font());
+	int w1 = fm.boundingRect(tr("Type")).width() + 26;
+	int w2 = fm.boundingRect(tr("Income")).width() + 10;
+	int w3 = fm.boundingRect(tr("Expense")).width() + 10;
+	if(w2 > w1) w1 = w2;
+	if(w3 > w1) w1 = w3;
+	transactionsView->setColumnWidth(0, w1);
+	setColumnStrlenWidth(transactionsView, 1, 15);
+	w1 = fm.boundingRect(tr("Account/Category")).width() + 26;
+	transactionsView->setColumnWidth(2, w1);
+	setColumnMoneyWidth(transactionsView, 3, budget);
 	transactionsView->setMinimumWidth(transactionsView->columnWidth(0) + transactionsView->columnWidth(1) + transactionsView->columnWidth(2) +  transactionsView->columnWidth(3) + 10);
 	box2->addWidget(transactionsView);
-	QDialogButtonBox *buttons = new QDialogButtonBox(0, Qt::Vertical);
-	QPushButton *newButton = buttons->addButton(tr("New"), QDialogButtonBox::ActionRole);
+	QVBoxLayout *buttons = new QVBoxLayout();
+	QPushButton *newButton = new QPushButton(tr("New"));
+	buttons->addWidget(newButton);
 	QMenu *newMenu = new QMenu(this);
 	newButton->setMenu(newMenu);
 	connect(newMenu->addAction(LOAD_ICON("document-new"), tr("New Expense…")), SIGNAL(triggered()), this, SLOT(newExpense()));
@@ -395,14 +422,31 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 	connect(newMenu->addAction(LOAD_ICON("document-new"), tr("New Securities Purchase…", "Financial security (e.g. stock, mutual fund)")), SIGNAL(triggered()), this, SLOT(newSecurityBuy()));
 	connect(newMenu->addAction(LOAD_ICON("document-new"), tr("New Securities Sale…", "Financial security (e.g. stock, mutual fund)")), SIGNAL(triggered()), this, SLOT(newSecuritySell()));
 	connect(newMenu->addAction(LOAD_ICON("document-new"), tr("New Dividend…")), SIGNAL(triggered()), this, SLOT(newDividend()));
-	editButton = buttons->addButton(tr("Edit…"), QDialogButtonBox::ActionRole);
+	editButton = new QPushButton(tr("Edit…"));
+	buttons->addWidget(editButton);
 	editButton->setEnabled(false);
-	removeButton = buttons->addButton(tr("Delete"), QDialogButtonBox::ActionRole);
+	removeButton = new QPushButton(tr("Delete"));
+	buttons->addWidget(removeButton);
+	buttons->addStretch(1);
 	removeButton->setEnabled(false);
-	box2->addWidget(buttons);
+	box2->addLayout(buttons);
 	totalLabel = new QLabel();
 	updateTotalValue();
 	box1->addWidget(totalLabel);
+	
+	connect(descriptionEdit, SIGNAL(returnPressed()), this, SLOT(focusDate()));
+	connect(dateEdit, SIGNAL(returnPressed()), accountCombo, SLOT(focusAndSelectAll()));
+	if(payeeEdit) {
+		connect(accountCombo, SIGNAL(accountSelected(Account*)), payeeEdit, SLOT(setFocus()));
+		connect(accountCombo, SIGNAL(returnPressed()), payeeEdit, SLOT(setFocus()));
+		connect(payeeEdit, SIGNAL(returnPressed()), tagButton, SLOT(setFocus()));
+	} else {
+		connect(accountCombo, SIGNAL(accountSelected(Account*)), tagButton, SLOT(setFocus()));
+		connect(accountCombo, SIGNAL(returnPressed()), tagButton, SLOT(setFocus()));
+	}
+	connect(tagButton, SIGNAL(returnPressed()), fileEdit, SLOT(setFocus()));
+	connect(fileEdit, SIGNAL(returnPressed()), commentEdit, SLOT(setFocus()));
+	connect(commentEdit, SIGNAL(returnPressed()), newButton, SLOT(setFocus()));
 
 	connect(selectFileButton, SIGNAL(clicked()), this, SLOT(selectFile()));
 	connect(openFileButton, SIGNAL(clicked()), this, SLOT(openFile()));
@@ -418,19 +462,32 @@ EditMultiItemWidget::EditMultiItemWidget(Budget *budg, QWidget *parent, AssetsAc
 EditMultiItemWidget::~EditMultiItemWidget() {}
 
 void EditMultiItemWidget::selectFile() {
-	QString url = QFileDialog::getOpenFileName(this, QString(), fileEdit->text().isEmpty() ? last_associated_file_directory : fileEdit->text());
-	if(!url.isEmpty()) {
-		QFileInfo fileInfo(url);
+	QStringList urls = QFileDialog::getOpenFileNames(this, QString(), (fileEdit->text().isEmpty() || fileEdit->text().contains(",")) ? last_associated_file_directory : fileEdit->text());
+	if(!urls.isEmpty()) {
+		QFileInfo fileInfo(urls[0]);
 		last_associated_file_directory = fileInfo.absoluteDir().absolutePath();
-		fileEdit->setText(url);
+		if(urls.size() == 1) {
+			fileEdit->setText(urls[0]);
+		} else {
+			QString url;
+			for(int i = 0; i < urls.size(); i++) {
+				if(i > 0) url += ", ";
+				if(urls[i].contains("\"")) {url += "\'"; url += urls[i]; url += "\'";}
+				else {url += "\""; url += urls[i]; url += "\"";}
+			}
+			fileEdit->setText(url);
+		}
 	}
 }
 void EditMultiItemWidget::openFile() {
-	QDesktopServices::openUrl(QUrl::fromLocalFile(fileEdit->text()));
+	open_file_list(fileEdit->text());
 }
 void EditMultiItemWidget::newAccount() {
 	accountCombo->createAccount();
 }
+void EditMultiItemWidget::newTag() {
+	tagButton->createTag();
+}
 void EditMultiItemWidget::accountChanged() {
 	Account *account = selectedAccount();
 	if(account) {
@@ -444,6 +501,11 @@ void EditMultiItemWidget::accountChanged() {
 		}
 	}
 }
+void EditMultiItemWidget::focusDate() {
+	if(!dateEdit) return;
+	dateEdit->setFocus();
+	dateEdit->setCurrentSection(QDateTimeEdit::DaySection);
+}
 
 void EditMultiItemWidget::updateTotalValue() {
 	double total_value = 0.0;
@@ -476,7 +538,7 @@ void EditMultiItemWidget::newTransaction(int transtype, bool select_security, bo
 	Currency *cur = budget->defaultCurrency();
 	if(selectedAccount()) cur = selectedAccount()->currency();
 	TransactionEditDialog *dialog = new TransactionEditDialog(b_extra, transtype, cur, transfer_to, NULL, SECURITY_ALL_VALUES, select_security, budget, this, b_create_accounts);
-	dialog->editWidget->focusDescription();
+	dialog->editWidget->focusFirst();
 	dialog->editWidget->updateAccounts(exclude_account);
 	if(dialog->editWidget->checkAccounts() && dialog->exec() == QDialog::Accepted) {
 		Transaction *trans = dialog->editWidget->createTransaction();
@@ -556,14 +618,17 @@ MultiItemTransaction *EditMultiItemWidget::createTransaction() {
 	MultiItemTransaction *split = new MultiItemTransaction(budget, dateEdit->date(), account, descriptionEdit->text());
 	if(payeeEdit) split->setPayee(payeeEdit->text());
 	if(fileEdit) split->setAssociatedFile(fileEdit->text());
+	split->setComment(commentEdit->text());
 	QTreeWidgetItemIterator it(transactionsView);
 	QTreeWidgetItem *i = *it;
+	if(tagButton) tagButton->modifyTransaction(split);
 	while(i) {
 		Transaction *trans = ((MultiItemListViewItem*) i)->transaction();
 		if(trans) split->addTransaction(trans);
 		++it;
 		i = *it;
 	}
+	split->joinTags();
 	return split;
 }
 void EditMultiItemWidget::setTransaction(MultiItemTransaction *split) {
@@ -572,11 +637,36 @@ void EditMultiItemWidget::setTransaction(MultiItemTransaction *split) {
 	accountCombo->setCurrentAccount(split->account());
 	if(payeeEdit) payeeEdit->setText(split->payee());
 	if(fileEdit) fileEdit->setText(split->associatedFile());
+	commentEdit->setText(split->comment());
 	transactionsView->clear();
 	QList<QTreeWidgetItem *> items;
+	tagButton->setTransaction(split);
 	int c = split->count();
+	QStringList tags;
+	for(int i = 0; i < c; i++) {
+		Transaction *trans = split->at(i);
+		if(i == 0) {
+			for(int i2 = 0; i2 < trans->tagsCount(false); i2++) {
+				tags << trans->getTag(i2, false);
+			}
+		} else {
+			for(int i2 = 0; i2 < tags.count();) {
+				if(!trans->hasTag(tags[i2], false)) {
+					tags.removeAt(i2);
+				} else {
+					i2++;
+				}
+			}
+		}
+	}
+	for(int i = 0; i < tags.count(); i++) {
+		tagButton->setTagSelected(tags[i], true);
+	}
 	for(int i = 0; i < c; i++) {
 		Transaction *trans = split->at(i)->copy();
+		for(int i2 = 0; i2 < tags.count(); i2++) {
+			trans->removeTag(tags[i2]);
+		}
 		trans->setAssociatedFile(QString());
 		trans->setDate(QDate());
 		items.append(new MultiItemListViewItem(trans, split->currency(), (trans->toAccount() == split->account())));
@@ -632,8 +722,8 @@ void EditMultiItemWidget::reject() {
 		i = *it;
 	}
 }
-void EditMultiItemWidget::focusDescription() {
-	return descriptionEdit->setFocus();
+void EditMultiItemWidget::focusFirst() {
+	descriptionEdit->setFocus();
 }
 QDate EditMultiItemWidget::date() {
 	return dateEdit->date();
@@ -697,7 +787,13 @@ EditMultiAccountWidget::EditMultiAccountWidget(Budget *budg, QWidget *parent, bo
 	categoryCombo->updateAccounts();
 	grid->addWidget(categoryCombo, b_extra ? 2 : 1, 1);
 	
-	grid->addWidget(new QLabel(tr("Associated file:"), this), b_extra ? 3 : 2, 0);
+	grid->addWidget(new QLabel(tr("Tags:")), b_extra ? 3 : 2, 0);
+	tagButton = new TagButton(false, allow_account_creation, budget, this);
+	grid->addWidget(tagButton, b_extra ? 3 : 2, 1);
+	tagButton->updateTags();
+	connect(tagButton, SIGNAL(newTagRequested()), this, SLOT(newTag()));
+	
+	grid->addWidget(new QLabel(tr("Associated file:"), this), b_extra ? 4 : 3, 0);
 	QHBoxLayout *fileLayout = new QHBoxLayout();
 	fileEdit = new QLineEdit(this);
 	QCompleter *completer = new QCompleter(this);
@@ -713,16 +809,16 @@ EditMultiAccountWidget::EditMultiAccountWidget(Budget *budg, QWidget *parent, bo
 	openFileButton->setAutoDefault(false);
 	fileLayout->addWidget(openFileButton);
 	openFileButton->setFocusPolicy(Qt::ClickFocus);
-	grid->addLayout(fileLayout, b_extra ? 3 : 2, 1);
+	grid->addLayout(fileLayout, b_extra ? 4 : 3, 1);
 	
-	grid->addWidget(new QLabel(tr("Comments:")), b_extra ? 4 : 3, 0);
+	grid->addWidget(new QLabel(tr("Comments:")), b_extra ? 5 : 4, 0);
 	commentEdit = new QLineEdit();
-	grid->addWidget(commentEdit, b_extra ? 4 : 3, 1);
+	grid->addWidget(commentEdit, b_extra ? 5 : 4, 1);
 
 	box1->addWidget(new QLabel(tr("Transactions:")));
 	QHBoxLayout *box2 = new QHBoxLayout();
 	box1->addLayout(box2);
-	transactionsView = new EqonomizeTreeWidget();
+	transactionsView = new EqonomizeTreeWidget(this);
 	transactionsView->setSortingEnabled(true);
 	transactionsView->sortByColumn(0, Qt::DescendingOrder);
 	transactionsView->setAllColumnsShowFocus(true);
@@ -736,19 +832,35 @@ EditMultiAccountWidget::EditMultiAccountWidget(Budget *budg, QWidget *parent, bo
 	transactionsView->setRootIsDecorated(false);
 	setColumnDateWidth(transactionsView, 0);
 	setColumnStrlenWidth(transactionsView, 1, 25);
-	setColumnMoneyWidth(transactionsView, 2);
+	setColumnMoneyWidth(transactionsView, 2, budget);
 	transactionsView->setMinimumWidth(transactionsView->columnWidth(0) + transactionsView->columnWidth(1) + transactionsView->columnWidth(2) + 10);
 	box2->addWidget(transactionsView);
-	QDialogButtonBox *buttons = new QDialogButtonBox(0, Qt::Vertical);
-	QPushButton *newButton = buttons->addButton(tr("New"), QDialogButtonBox::ActionRole);
-	editButton = buttons->addButton(tr("Edit…"), QDialogButtonBox::ActionRole);
+	QVBoxLayout *buttons = new QVBoxLayout();
+	QPushButton *newButton = new QPushButton(tr("New"));
+	buttons->addWidget(newButton);
+	editButton = new QPushButton(tr("Edit…"));
+	buttons->addWidget(editButton);
 	editButton->setEnabled(false);
-	removeButton = buttons->addButton(tr("Delete"), QDialogButtonBox::ActionRole);
+	removeButton = new QPushButton(tr("Delete"));
+	buttons->addWidget(removeButton);
+	buttons->addStretch(1);
 	removeButton->setEnabled(false);
-	box2->addWidget(buttons);
+	box2->addLayout(buttons);
 	totalLabel = new QLabel();
 	updateTotalValue();
 	box1->addWidget(totalLabel);
+	
+	if(quantityEdit) {
+		connect(descriptionEdit, SIGNAL(returnPressed()), quantityEdit, SLOT(enterFocus()));
+		connect(quantityEdit, SIGNAL(returnPressed()), categoryCombo, SLOT(focusAndSelectAll()));
+	} else {
+		connect(descriptionEdit, SIGNAL(returnPressed()), categoryCombo, SLOT(focusAndSelectAll()));
+	}
+	connect(categoryCombo, SIGNAL(accountSelected(Account*)), fileEdit, SLOT(setFocus()));
+	connect(categoryCombo, SIGNAL(returnPressed()), tagButton, SLOT(setFocus()));
+	connect(tagButton, SIGNAL(returnPressed()), fileEdit, SLOT(setFocus()));
+	connect(fileEdit, SIGNAL(returnPressed()), commentEdit, SLOT(setFocus()));
+	connect(commentEdit, SIGNAL(returnPressed()), newButton, SLOT(setFocus()));
 
 	connect(selectFileButton, SIGNAL(clicked()), this, SLOT(selectFile()));
 	connect(openFileButton, SIGNAL(clicked()), this, SLOT(openFile()));
@@ -763,15 +875,25 @@ EditMultiAccountWidget::EditMultiAccountWidget(Budget *budg, QWidget *parent, bo
 EditMultiAccountWidget::~EditMultiAccountWidget() {}
 
 void EditMultiAccountWidget::selectFile() {
-	QString url = QFileDialog::getOpenFileName(this, QString(), fileEdit->text().isEmpty() ? last_associated_file_directory : fileEdit->text());
-	if(!url.isEmpty()) {
-		QFileInfo fileInfo(url);
+	QStringList urls = QFileDialog::getOpenFileNames(this, QString(), (fileEdit->text().isEmpty() || fileEdit->text().contains(",")) ? last_associated_file_directory : fileEdit->text());
+	if(!urls.isEmpty()) {
+		QFileInfo fileInfo(urls[0]);
 		last_associated_file_directory = fileInfo.absoluteDir().absolutePath();
-		fileEdit->setText(url);
+		if(urls.size() == 1) {
+			fileEdit->setText(urls[0]);
+		} else {
+			QString url;
+			for(int i = 0; i < urls.size(); i++) {
+				if(i > 0) url += ", ";
+				if(urls[i].contains("\"")) {url += "\'"; url += urls[i]; url += "\'";}
+				else {url += "\""; url += urls[i]; url += "\"";}
+			}
+			fileEdit->setText(url);
+		}
 	}
 }
 void EditMultiAccountWidget::openFile() {
-	QDesktopServices::openUrl(QUrl::fromLocalFile(fileEdit->text()));
+	open_file_list(fileEdit->text());
 }
 
 void EditMultiAccountWidget::newCategory() {
@@ -819,7 +941,7 @@ void EditMultiAccountWidget::transactionSelectionChanged() {
 }
 void EditMultiAccountWidget::newTransaction() {
 	TransactionEditDialog *dialog = new TransactionEditDialog(b_extra, b_expense ? TRANSACTION_TYPE_EXPENSE : TRANSACTION_TYPE_INCOME, NULL, false, NULL, SECURITY_ALL_VALUES, false, budget, this, b_create_accounts, true);
-	dialog->editWidget->focusDescription();
+	dialog->editWidget->focusFirst();
 	dialog->editWidget->updateAccounts();
 	if(dialog->editWidget->checkAccounts() && dialog->exec() == QDialog::Accepted) {
 		Transaction *trans = dialog->editWidget->createTransaction();
@@ -868,12 +990,16 @@ void EditMultiAccountWidget::edit(QTreeWidgetItem *i_pre) {
 		dialog->deleteLater();
 	}
 }
+void EditMultiAccountWidget::newTag() {
+	tagButton->createTag();
+}
 MultiAccountTransaction *EditMultiAccountWidget::createTransaction() {
 	if(!validValues()) return NULL;
 	CategoryAccount *account = selectedCategory();
 	MultiAccountTransaction *split = new MultiAccountTransaction(budget, account, descriptionEdit->text());
 	split->setComment(commentEdit->text());
 	split->setAssociatedFile(fileEdit->text());
+	tagButton->modifyTransaction(split);
 	QTreeWidgetItemIterator it(transactionsView);
 	QTreeWidgetItem *i = *it;
 	while(i) {
@@ -920,6 +1046,7 @@ void EditMultiAccountWidget::setTransaction(Transactions *transs) {
 			categoryCombo->setCurrentAccount(((Income*) trans)->category());
 		}
 	}
+	tagButton->setTransaction(transs);
 	updateTotalValue();
 	transactionsView->setSortingEnabled(true);
 	emit dateChanged(transs->date());
@@ -940,6 +1067,7 @@ void EditMultiAccountWidget::setTransaction(MultiAccountTransaction *split, cons
 		trans->setDate(date);
 		items.append(new MultiAccountListViewItem(trans));
 	}
+	tagButton->setTransaction(split);
 	transactionsView->addTopLevelItems(items);
 	updateTotalValue();
 	transactionsView->setSortingEnabled(true);
@@ -977,7 +1105,7 @@ void EditMultiAccountWidget::reject() {
 		i = *it;
 	}
 }
-void EditMultiAccountWidget::focusDescription() {
+void EditMultiAccountWidget::focusFirst() {
 	return descriptionEdit->setFocus();
 }
 bool EditMultiAccountWidget::checkAccounts() {
@@ -1015,7 +1143,7 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 	grid->addWidget(loanCombo, row, 1); row++;
 
 	grid->addWidget(new QLabel(tr("Date:")), row, 0);
-	dateEdit = new QDateEdit(QDate::currentDate());
+	dateEdit = new EqonomizeDateEdit(this);
 	dateEdit->setCalendarPopup(true);
 	grid->addWidget(dateEdit, row, 1); row++;
 	
@@ -1045,16 +1173,16 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 		totalLabel = NULL;
 		accountCombo = NULL;
 		addedInterestButton = NULL;
-		payedInterestButton = NULL;
+		paidInterestButton = NULL;
 	} else {
-		QHBoxLayout *payedAddedLayout = new QHBoxLayout();
-		payedAddedLayout->addStretch(1);
-		payedInterestButton = new QRadioButton(tr("Payed"), this);
-		payedAddedLayout->addWidget(payedInterestButton);
-		addedInterestButton = new QRadioButton(tr("Added to debt"), this);
-		payedAddedLayout->addWidget(addedInterestButton);
-		payedInterestButton->setChecked(true);
-		grid->addLayout(payedAddedLayout, row, 0, 1, 2); row++;
+		QHBoxLayout *paidAddedLayout = new QHBoxLayout();
+		paidAddedLayout->addStretch(1);
+		paidInterestButton = new EqonomizeRadioButton(tr("Paid"), this);
+		paidAddedLayout->addWidget(paidInterestButton);
+		addedInterestButton = new EqonomizeRadioButton(tr("Added to debt"), this);
+		paidAddedLayout->addWidget(addedInterestButton);
+		paidInterestButton->setChecked(true);
+		grid->addLayout(paidAddedLayout, row, 0, 1, 2); row++;
 		
 		grid->addWidget(new QLabel(tr("Fee:")), row, 0);
 		feeEdit = new EqonomizeValueEdit(false, this, budget);
@@ -1102,6 +1230,8 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 		commentEdit = new QLineEdit();
 		grid->addWidget(commentEdit, row, 1); row++;
 	}
+	
+	loanCombo->focusAndSelectAll();
 
 	if(default_loan) loanCombo->setCurrentAccount(default_loan);
 
@@ -1111,6 +1241,28 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 	valueChanged();
 	interestSourceChanged();
 	b_search = true;
+	
+	connect(loanCombo, SIGNAL(returnPressed()), this, SLOT(focusDate()));
+	connect(loanCombo, SIGNAL(accountSelected(Account*)), this, SLOT(focusDate()));
+	if(only_interest) {
+		connect(dateEdit, SIGNAL(returnPressed()), interestEdit, SLOT(enterFocus()));
+		connect(interestEdit, SIGNAL(returnPressed()), categoryCombo, SLOT(focusAndSelectAll()));
+		connect(categoryCombo, SIGNAL(returnPressed()), this, SIGNAL(addmodify()));
+	} else {
+		connect(dateEdit, SIGNAL(returnPressed()), reductionEdit, SLOT(enterFocus()));
+		connect(reductionEdit, SIGNAL(returnPressed()), this, SLOT(reductionFocusNext()));
+		connect(paymentEdit, SIGNAL(returnPressed()), interestEdit, SLOT(enterFocus()));
+		connect(interestEdit, SIGNAL(returnPressed()), paidInterestButton, SLOT(setFocus()));
+		connect(paidInterestButton, SIGNAL(returnPressed()), feeEdit, SLOT(enterFocus()));
+		connect(addedInterestButton, SIGNAL(returnPressed()), feeEdit, SLOT(enterFocus()));
+		connect(feeEdit, SIGNAL(returnPressed()), this, SLOT(feeFocusNext()));
+		connect(accountCombo, SIGNAL(accountSelected(Account*)), this, SLOT(accountFocusNext()));
+		connect(accountCombo, SIGNAL(returnPressed()), this, SLOT(accountFocusNext()));
+		connect(categoryCombo, SIGNAL(accountSelected(Account*)), fileEdit, SLOT(setFocus()));
+		connect(categoryCombo, SIGNAL(returnPressed()), fileEdit, SLOT(setFocus()));
+		connect(fileEdit, SIGNAL(returnPressed()), commentEdit, SLOT(setFocus()));
+		connect(commentEdit, SIGNAL(returnPressed()), this, SIGNAL(addmodify()));
+	}
 
 	if(dateEdit) connect(dateEdit, SIGNAL(dateChanged(const QDate&)), this, SIGNAL(dateChanged(const QDate&)));
 	if(dateEdit) connect(dateEdit, SIGNAL(dateChanged(const QDate&)), this, SLOT(hasBeenModified()));
@@ -1121,7 +1273,7 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 	if(reductionEdit && paymentEdit) connect(reductionEdit, SIGNAL(editingFinished()), this, SLOT(reductionEditingFinished()));
 	if(reductionEdit) connect(reductionEdit, SIGNAL(valueChanged(double)), this, SLOT(valueChanged()));
 	if(interestEdit) connect(interestEdit, SIGNAL(valueChanged(double)), this, SLOT(valueChanged()));
-	if(payedInterestButton) connect(payedInterestButton, SIGNAL(toggled(bool)), this, SLOT(interestSourceChanged()));
+	if(paidInterestButton) connect(paidInterestButton, SIGNAL(toggled(bool)), this, SLOT(interestSourceChanged()));
 	if(addedInterestButton) connect(addedInterestButton, SIGNAL(toggled(bool)), this, SLOT(interestSourceChanged()));
 	if(loanCombo) connect(loanCombo, SIGNAL(accountSelected(Account*)), this, SLOT(loanActivated(Account*)));
 	if(accountCombo) connect(accountCombo, SIGNAL(newAccountRequested()), this, SLOT(newAccount()));
@@ -1132,16 +1284,48 @@ EditDebtPaymentWidget::EditDebtPaymentWidget(Budget *budg, QWidget *parent, Asse
 }
 EditDebtPaymentWidget::~EditDebtPaymentWidget() {}
 
+void EditDebtPaymentWidget::focusFirst() {
+	loanCombo->focusAndSelectAll();
+}
+void EditDebtPaymentWidget::focusDate() {
+	if(!dateEdit) return;
+	dateEdit->setFocus();
+	dateEdit->setCurrentSection(QDateTimeEdit::DaySection);
+}
+void EditDebtPaymentWidget::reductionFocusNext() {
+	if(paymentEdit->isVisible() && paymentEdit->isEnabled()) paymentEdit->enterFocus();
+	else interestEdit->enterFocus();
+}
+void EditDebtPaymentWidget::accountFocusNext() {
+	if(categoryCombo->isEnabled()) categoryCombo->focusAndSelectAll();
+	else fileEdit->setFocus();
+}
+void EditDebtPaymentWidget::feeFocusNext() {
+	if(accountCombo->isEnabled()) accountCombo->focusAndSelectAll();
+	else if(categoryCombo->isEnabled()) categoryCombo->focusAndSelectAll();
+	else fileEdit->setFocus();
+}
+
 void EditDebtPaymentWidget::selectFile() {
-	QString url = QFileDialog::getOpenFileName(this, QString(), fileEdit->text().isEmpty() ? last_associated_file_directory : fileEdit->text());
-	if(!url.isEmpty()) {
-		QFileInfo fileInfo(url);
+	QStringList urls = QFileDialog::getOpenFileNames(this, QString(), (fileEdit->text().isEmpty() || fileEdit->text().contains(",")) ? last_associated_file_directory : fileEdit->text());
+	if(!urls.isEmpty()) {
+		QFileInfo fileInfo(urls[0]);
 		last_associated_file_directory = fileInfo.absoluteDir().absolutePath();
-		fileEdit->setText(url);
+		if(urls.size() == 1) {
+			fileEdit->setText(urls[0]);
+		} else {
+			QString url;
+			for(int i = 0; i < urls.size(); i++) {
+				if(i > 0) url += ", ";
+				if(urls[i].contains("\"")) {url += "\'"; url += urls[i]; url += "\'";}
+				else {url += "\""; url += urls[i]; url += "\"";}
+			}
+			fileEdit->setText(url);
+		}
 	}
 }
 void EditDebtPaymentWidget::openFile() {
-	QDesktopServices::openUrl(QUrl::fromLocalFile(fileEdit->text()));
+	open_file_list(fileEdit->text());
 }
 
 void EditDebtPaymentWidget::newAccount() {
@@ -1226,8 +1410,8 @@ void EditDebtPaymentWidget::reductionEditingFinished() {
 }
 void EditDebtPaymentWidget::valueChanged() {
 	if(categoryCombo) categoryCombo->setEnabled((!paymentEdit && !reductionEdit) || (interestEdit && interestEdit->value() > 0.0) || (feeEdit && feeEdit->value() > 0.0));
-	if(accountCombo && interestEdit && payedInterestButton) {
-		accountCombo->setEnabled(interestEdit->value() == 0.0 || (feeEdit && feeEdit->value() > 0.0) || (paymentEdit && paymentEdit->value() > 0.0) || (reductionEdit && reductionEdit->value() > 0.0) || payedInterestButton->isChecked());
+	if(accountCombo && interestEdit && paidInterestButton) {
+		accountCombo->setEnabled(interestEdit->value() == 0.0 || (feeEdit && feeEdit->value() > 0.0) || (paymentEdit && paymentEdit->value() > 0.0) || (reductionEdit && reductionEdit->value() > 0.0) || paidInterestButton->isChecked());
 	}
 	Account *acc = selectedAccount();
 	AssetsAccount *loan = selectedLoan();
@@ -1342,7 +1526,7 @@ void EditDebtPaymentWidget::setTransaction(DebtPayment *split) {
 	if(paymentEdit) paymentEdit->setValue(split->payment());
 	if(reductionEdit) reductionEdit->setValue(split->reduction());
 	if(interestEdit) interestEdit->setValue(split->interest());
-	if(payedInterestButton && split->interestPayed()) payedInterestButton->setChecked(true);
+	if(paidInterestButton && split->interestPaid()) paidInterestButton->setChecked(true);
 	else if(addedInterestButton) addedInterestButton->setChecked(true);
 	if(feeEdit) feeEdit->setValue(split->fee());
 	valueChanged();
