@@ -24,6 +24,7 @@
 
 #include <QWidget>
 #include <QDialog>
+#include <QRadioButton>
 
 class QLabel;
 class QPushButton;
@@ -32,9 +33,9 @@ class QTreeWidgetItem;
 class QLineEdit;
 class QComboBox;
 class QDateEdit;
-class QRadioButton;
 
 class EqonomizeValueEdit;
+class TagButton;
 
 class Account;
 class AccountComboBox;
@@ -63,6 +64,7 @@ class EditMultiAccountWidget : public QWidget {
 		QTreeWidget *transactionsView;
 		QPushButton *editButton, *removeButton;
 		QLabel *totalLabel;
+		TagButton *tagButton;
 
 		void appendTransaction(Transaction *trans);
 		void updateTotalValue();
@@ -79,7 +81,7 @@ class EditMultiAccountWidget : public QWidget {
 		void setTransaction(MultiAccountTransaction *split, const QDate &date);
 		bool validValues();
 		bool checkAccounts();
-		void focusDescription();
+		void focusFirst();
 		void reject();
 		QDate date();
 		
@@ -97,6 +99,7 @@ class EditMultiAccountWidget : public QWidget {
 		void transactionSelectionChanged();
 		void newTransaction();
 		void newCategory();
+		void newTag();
 
 };
 
@@ -111,7 +114,7 @@ class EditDebtPaymentWidget : public QWidget {
 		
 		QDateEdit *dateEdit;
 		EqonomizeValueEdit *reductionEdit, *paymentEdit, *interestEdit, *feeEdit;
-		QRadioButton *payedInterestButton, *addedInterestButton;
+		QRadioButton *paidInterestButton, *addedInterestButton;
 		AccountComboBox *accountCombo, *categoryCombo, *loanCombo;
 		QLineEdit *commentEdit, *fileEdit;
 		QLabel *paymentLabel, *totalLabel;
@@ -132,10 +135,12 @@ class EditDebtPaymentWidget : public QWidget {
 		bool validValues();
 		bool checkAccounts();
 		QDate date();
+		void focusFirst();
 	
 	signals:
 	
 		void dateChanged(const QDate &d);
+		void addmodify();
 
 	protected slots:
 		
@@ -150,6 +155,10 @@ class EditDebtPaymentWidget : public QWidget {
 		void reductionEditingFinished();
 		void interestSourceChanged();
 		void hasBeenModified();
+		void focusDate();
+		void reductionFocusNext();
+		void accountFocusNext();
+		void feeFocusNext();
 
 };
 
@@ -164,10 +173,11 @@ class EditMultiItemWidget : public QWidget {
 		
 		QDateEdit *dateEdit;
 		AccountComboBox *accountCombo;
-		QLineEdit *descriptionEdit, *payeeEdit, *fileEdit;
+		QLineEdit *descriptionEdit, *payeeEdit, *fileEdit, *commentEdit;
 		QTreeWidget *transactionsView;
 		QPushButton *editButton, *removeButton;
 		QLabel *totalLabel;
+		TagButton *tagButton;
 
 		void appendTransaction(Transaction *trans, bool deposit);
 		void newTransaction(int transtype, bool select_security = false, bool transfer_to = false, Account *exclude_account = NULL);
@@ -185,7 +195,7 @@ class EditMultiItemWidget : public QWidget {
 		bool validValues();
 		bool checkAccounts();
 		void reject();
-		void focusDescription();
+		void focusFirst();
 		QDate date();
 		
 	signals:
@@ -209,6 +219,8 @@ class EditMultiItemWidget : public QWidget {
 		void newTransferFrom();
 		void newTransferTo();
 		void newAccount();
+		void focusDate();
+		void newTag();
 
 };
 
@@ -222,6 +234,10 @@ class EditMultiItemDialog : public QDialog {
 		~EditMultiItemDialog();
 		
 		EditMultiItemWidget *editWidget;
+		
+	protected:
+	
+		void keyPressEvent(QKeyEvent*);
 
 	protected slots:
 		
@@ -241,6 +257,10 @@ class EditMultiAccountDialog : public QDialog {
 		
 		EditMultiAccountWidget *editWidget;
 
+	protected:
+	
+		void keyPressEvent(QKeyEvent*);
+
 	protected slots:
 		
 		void accept();
@@ -259,12 +279,33 @@ class EditDebtPaymentDialog : public QDialog {
 		
 		EditDebtPaymentWidget *editWidget;
 
+	protected:
+	
+		void keyPressEvent(QKeyEvent*);
+
 	protected slots:
 		
 		void accept();
 
 };
 
+class EqonomizeRadioButton : public QRadioButton {
+
+	Q_OBJECT
+
+	public:
+	
+		EqonomizeRadioButton(const QString &text, QWidget *parent);
+	
+	protected slots:
+	
+		void keyPressEvent(QKeyEvent *event);
+		
+	signals:
+	
+		void returnPressed();
+
+};
 
 #endif
 
