@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   Copyright (C) 2006-2008, 2014, 2016-2019 by Hanna Knutsson            *
  *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
  *   This file is part of Eqonomize!.                                      *
@@ -21,11 +21,13 @@
 #ifndef EQONOMIZE_H
 #define EQONOMIZE_H
 
+#include <QComboBox>
 #include <QDateTime>
 #include <QDialog>
 #include <QDragEnterEvent>
 #include <QDropEvent>
 #include <QLabel>
+#include <QHash>
 #include <QMap>
 #include <QModelIndex>
 #include <QTextStream>
@@ -36,22 +38,28 @@
 #include <QUrl>
 #include <QApplication>
 #include <QMainWindow>
+#include <QStyledItemDelegate>
+#include <QTranslator>
 
 #ifdef LOAD_EQZICONS_FROM_FILE
 	#ifdef RESOURCES_COMPILED
 		#define LOAD_APP_ICON(x) QIcon(ICON_DIR "/EQZ/apps/64x64/" x ".png")
-		/*#define LOAD_ICON(x) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png") : QIcon::fromTheme(x))
-		#define LOAD_ICON2(x, y) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png") : QIcon::fromTheme(x, y))*/
 		#define LOAD_ICON(x) QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png")
+		#define LOAD_ICON_APP(x) QIcon(ICON_DIR "/EQZ/apps/64x64/" x ".png")
+		#define LOAD_ICON_STATUS(x) QIcon(ICON_DIR "/EQZ/status/64x64/" x ".png")
 		#define LOAD_ICON2(x, y) QIcon(ICON_DIR "/EQZ/actions/64x64/" y ".png")
 	#else
 		#define LOAD_APP_ICON(x) QIcon(ICON_DIR "/hicolor/64x64/apps/" x ".png")
 		#define LOAD_ICON(x) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/hicolor/64x64/actions/" x ".png") : QIcon::fromTheme(x))
+		#define LOAD_ICON_APP(x) QIcon::fromTheme(x)
+		#define LOAD_ICON_STATUS(x) QIcon::fromTheme(x)
 		#define LOAD_ICON2(x, y) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/hicolor/64x64/actions/" x ".png") : QIcon::fromTheme(x, QIcon::fromTheme(y)))
 	#endif
 #else
 	#define LOAD_APP_ICON(x) QIcon::fromTheme(x)
 	#define LOAD_ICON(x) QIcon::fromTheme(x)
+	#define LOAD_ICON_APP(x) QIcon::fromTheme(x)
+	#define LOAD_ICON_STATUS(x) QIcon::fromTheme(x)
 	#define LOAD_ICON2(x, y) QIcon::fromTheme(x, QIcon::fromTheme(y))
 #endif
 
@@ -65,7 +73,6 @@ class QRadioButton;
 class QSpinBox;
 class QTabWidget;
 class QVBoxLayout;
-class QComboBox;
 class QLineEdit;
 class QCommandLineParser;
 class QToolBar;
@@ -77,6 +84,7 @@ class QLocalServer;
 class QPrinter;
 class QDialog;
 class QNetworkReply;
+class TagMenu;
 
 class CategoriesComparisonChart;
 class CategoriesComparisonReport;
@@ -141,11 +149,14 @@ class Eqonomize : public QMainWindow {
 		void addTransactionValue(Transaction *trans, const QDate &transdate, bool update_value_display, bool subtract = false, int n = -1, int b_future = -1, const QDate *monthdate = NULL);
 		void appendIncomesAccount(IncomesAccount *account, QTreeWidgetItem *parent_item);
 		void appendExpensesAccount(ExpensesAccount *account, QTreeWidgetItem *parent_item);
+		void assetsAccountItemHiddenOrRemoved(AssetsAccount *account);
+		void assetsAccountItemShownOrAdded(AssetsAccount *account);
 		void appendAssetsAccount(AssetsAccount *account);
 		void appendLoanAccount(LoanAccount *account);
 		void updateMonthlyBudget(Account *account);
 		void updateTotalMonthlyExpensesBudget();
 		void updateTotalMonthlyIncomesBudget();
+		void updateBudgetAccountTitle(AssetsAccount *account = NULL);
 		bool editAccount(Account*);
 		bool editAccount(Account*, QWidget *parent);
 		void balanceAccount(Account*);
@@ -153,10 +164,10 @@ class Eqonomize : public QMainWindow {
 		void updateScheduledTransactions();
 		void appendScheduledTransaction(ScheduledTransaction *strans);
 		bool editScheduledTransaction(ScheduledTransaction *strans);
-		bool editScheduledTransaction(ScheduledTransaction *strans, QWidget *parent);
+		bool editScheduledTransaction(ScheduledTransaction *strans, QWidget *parent, bool clone_trans = false);
 		bool editOccurrence(ScheduledTransaction *strans, const QDate &date);
 		bool editOccurrence(ScheduledTransaction *strans, const QDate &date, QWidget *parent);
-		bool editTransaction(Transaction *trans, QWidget *parent);
+		bool editTransaction(Transaction *trans, QWidget *parent, bool clone_trans = false);
 		bool editTransaction(Transaction *trans);
 		bool removeScheduledTransaction(ScheduledTransaction *strans);
 		bool removeOccurrence(ScheduledTransaction *strans, const QDate &date);
@@ -168,8 +179,9 @@ class Eqonomize : public QMainWindow {
 		bool newMultiAccountTransaction(QWidget *parent, bool create_expenses);
 		bool newMultiItemTransaction(QWidget *parent, AssetsAccount *account = NULL);
 		bool newDebtPayment(QWidget *parent, AssetsAccount *loan = NULL, bool only_interest = false);
+		bool editTimestamp(QList<Transactions*> trans, QWidget *parent = NULL);
 		bool editSplitTransaction(SplitTransaction *split);
-		bool editSplitTransaction(SplitTransaction *split, QWidget *parent, bool temporary_split = false);
+		bool editSplitTransaction(SplitTransaction *split, QWidget *parent, bool temporary_split = false, bool clone_trans = false);
 		bool splitUpTransaction(SplitTransaction *split);
 		bool removeSplitTransaction(SplitTransaction *split);
 		bool saveView(QTextStream &file, int fileformat);
@@ -203,6 +215,9 @@ class Eqonomize : public QMainWindow {
 		QAction *ActionNewMultiAccountExpense, *ActionNewExpenseWithLoan, *ActionNewDebtPayment, *ActionNewDebtInterest;
 		QAction *ActionEditTransaction, *ActionEditScheduledTransaction, *ActionEditSplitTransaction;
 		QAction *ActionJoinTransactions, *ActionSplitUpTransaction;
+		QAction *ActionCloneTransaction;
+		QAction *ActionEditTimestamp;
+		QAction *ActionTags;
 		QAction *ActionSelectAssociatedFile, *ActionOpenAssociatedFile;
 		QAction *ActionDeleteTransaction, *ActionDeleteScheduledTransaction, *ActionDeleteSplitTransaction;
 		QAction *ActionNewSecurity, *ActionEditSecurity, *ActionBuyShares, *ActionSellShares, *ActionNewDividend, *ActionNewReinvestedDividend, *ActionNewSecurityTrade, *ActionSetQuotation, *ActionEditQuotations, *ActionEditSecurityTransactions, *ActionDeleteSecurity;
@@ -214,10 +229,13 @@ class Eqonomize : public QMainWindow {
 		QAction *ActionOverTimeReport, *ActionCategoriesComparisonReport, *ActionOverTimeChart, *ActionCategoriesComparisonChart;
 		QAction *ActionImportCSV, *ActionImportQIF, *ActionImportEQZ, *ActionExportQIF;
 		QAction *ActionConvertCurrencies, *ActionUpdateExchangeRates;
-		QAction *ActionExtraProperties, *ActionUseExchangeRateForTransactionDate, *ActionSetBudgetPeriod, *AIPCurrentMonth, *AIPCurrentYear, *AIPCurrentWholeMonth, *AIPCurrentWholeYear, *AIPRememberLastDates, *ABFDaily, *ABFWeekly, *ABFFortnightly, *ABFMonthly, *ABFNever;
-		QAction *ActionSetMainCurrency, *ActionSyncSettings;
-		QActionGroup *ActionSelectInitialPeriod, *ActionSelectBackupFrequency;
+		QAction *ActionExtraProperties, *ActionUseExchangeRateForTransactionDate, *ActionSetBudgetPeriod, *ActionSetScheduleConfirmationTime, *AIPCurrentMonth, *AIPCurrentYear, *AIPCurrentWholeMonth, *AIPCurrentWholeYear, *AIPRememberLastDates, *ABFDaily, *ABFWeekly, *ABFFortnightly, *ABFMonthly, *ABFNever, *ACSTime[11];
+		QAction *ActionSetMainCurrency, *ActionSyncSettings, *ActionSelectFont;
+		QActionGroup *ActionSelectInitialPeriod, *ActionSelectBackupFrequency, *ActionSelectLang;
 		QAction *ActionHelp, *ActionWhatsThis, *ActionReportBug, *ActionAbout, *ActionAboutQt;
+		QAction *ActionNewTag, *ActionRenameTag, *ActionRemoveTag;
+		
+		TagMenu *tagMenu;
 		
 	protected:
 
@@ -252,7 +270,7 @@ class Eqonomize : public QMainWindow {
 		QVBoxLayout *expensesLayout, *incomesLayout, *transfersLayout;
 		TransactionListWidget *expensesWidget, *incomesWidget, *transfersWidget;
 		QTreeWidget *accountsView, *securitiesView, *scheduleView;
-		QTreeWidgetItem *assetsItem, *liabilitiesItem, *incomesItem, *expensesItem;
+		QTreeWidgetItem *assetsItem, *liabilitiesItem, *incomesItem, *expensesItem, *tagsItem;
 		QCheckBox *accountsPeriodFromButton;
 		QDateEdit *accountsPeriodFromEdit, *accountsPeriodToEdit;
 		QCheckBox *partialBudgetButton;
@@ -276,6 +294,12 @@ class Eqonomize : public QMainWindow {
 		double expenses_budget, expenses_budget_diff, incomes_budget, incomes_budget_diff;
 		QMap<Account*, double> account_value;
 		QMap<Account*, double> account_change;
+		QMap<QString, double> assets_group_value;
+		QMap<QString, double> assets_group_change;
+		QMap<QString, double> liabilities_group_value;
+		QMap<QString, double> liabilities_group_change;
+		QMap<QString, double> tag_value;
+		QMap<QString, double> tag_change;
 		QMap<Account*, QMap<QDate, double> > account_month;
 		QMap<Account*, double> account_month_begincur;
 		QMap<Account*, double> account_month_beginfirst;
@@ -285,19 +309,24 @@ class Eqonomize : public QMainWindow {
 		QMap<Account*, double> account_future_diff;
 		QMap<Account*, double> account_future_diff_change;
 		QMap<QTreeWidgetItem*, Account*> account_items;
+		QMap<QTreeWidgetItem*, QString> assets_group_items, liabilities_group_items, tag_items;
 		QMap<Account*, QTreeWidgetItem*> item_accounts;
+		QMap<QString, QTreeWidgetItem*> item_assets_groups, item_liabilities_groups, item_tags;
+		QMap<QString, QVariant> assets_expanded, liabilities_expanded, expenses_expanded, incomes_expanded;
 
-		QMenu *assetsPopupMenu, *accountPopupMenu, *securitiesPopupMenu, *schedulePopupMenu;
+		QMenu *assetsPopupMenu, *accountPopupMenu, *tagPopupMenu, *securitiesPopupMenu, *schedulePopupMenu, *scheduleHeaderPopupMenu;
 		
 		QDialog *helpDialog, *cccDialog, *ccrDialog, *otcDialog, *otrDialog, *syncDialog;
 		
 		QLineEdit *syncUploadEdit, *syncUrlEdit, *syncDownloadEdit;
 		QPushButton *uploadButton;
 		QCheckBox *syncAutoBox;
+		QTreeWidgetItem *clicked_item;
 		
 	protected slots:
 	
 		void checkAvailableVersion_readdata();
+		void languageSelected();
 
 	public slots:
 
@@ -314,6 +343,10 @@ class Eqonomize : public QMainWindow {
 		void updateBudgetDay();
 		void setBudgetPeriod();
 		
+		void setScheduleConfirmationTime();
+		
+		void selectFont();
+		
 		void showFilter();
 		
 		void showHelp();
@@ -385,15 +418,20 @@ class Eqonomize : public QMainWindow {
 		void newScheduledIncome();
 		void newScheduledTransfer();
 		void editScheduledTransaction();
+		void cloneScheduledTransaction();
 		void editOccurrence();
 		void removeScheduledTransaction();
 		void removeOccurrence();
 		void scheduleSelectionChanged();
 		void scheduleExecuted(QTreeWidgetItem*);
+		void hideScheduleColumn(bool);
+		void popupScheduleHeaderMenu(const QPoint&);
 		void popupScheduleMenu(const QPoint&);
 
+		void editSelectedTimestamp();
 		void editSelectedScheduledTransaction();
 		void editSelectedTransaction();
+		void cloneSelectedTransaction();
 		void editSelectedSplitTransaction();
 		void deleteSelectedScheduledTransaction();
 		void deleteSelectedTransaction();
@@ -442,13 +480,23 @@ class Eqonomize : public QMainWindow {
 		void newIncomesAccount(IncomesAccount *default_parent = NULL);
 		void newExpensesAccount(ExpensesAccount *default_parent = NULL);
 		void accountExecuted(QTreeWidgetItem*, int);
+		void accountClicked(QTreeWidgetItem*, int);
+		void accountClickedTimeout();
 		void accountExecuted(QTreeWidgetItem*);
+		void accountExpanded(QTreeWidgetItem*);
+		void accountCollapsed(QTreeWidgetItem*);
 		void accountMoved(QTreeWidgetItem*, QTreeWidgetItem*);
 		void balanceAccount();
 		void editAccount();
 		void deleteAccount();
 		void closeAccount();
 		void accountsSelectionChanged();
+		
+		void tagAdded(QString);
+		void newTag();
+		void deleteTag();
+		void renameTag();
+		void tagsChanged();
 
 		void setPartialBudget(bool);
 
@@ -479,7 +527,7 @@ class Eqonomize : public QMainWindow {
 		
 		void transactionAdded(Transactions*);
 		void transactionModified(Transactions*, Transactions*);
-		void transactionRemoved(Transactions*);
+		void transactionRemoved(Transactions*, Transactions* = NULL);
 
 		void filterAccounts();
 
@@ -489,6 +537,7 @@ class Eqonomize : public QMainWindow {
 		void transactionsModified();
 		void budgetUpdated();
 		void timeToSaveConfig();
+		void tagsModified();
 
 };
 
@@ -615,7 +664,9 @@ class EditSecurityDialog : public QDialog {
 		void newAccount();
 
 };
- 
+
+struct q_csv_info;
+
 class EditQuotationsDialog : public QDialog {
 
 	Q_OBJECT
@@ -629,13 +680,16 @@ class EditQuotationsDialog : public QDialog {
 		QPushButton *changeButton, *addButton, *deleteButton;
 		int i_quotation_decimals;
 		Budget *budget;
+		Security *security;
+		
+		bool import(QString url, bool test, q_csv_info *ci);
 
 	public:
 
-		EditQuotationsDialog(Budget *budg, QWidget *parent);
+		EditQuotationsDialog(Security *sec, QWidget *parent);
 
-		void setSecurity(Security *security);
-		void modifyQuotations(Security *security);
+		void setSecurity(Security *sec);
+		void modifyQuotations();
 
 	protected slots:
 		
@@ -643,6 +697,9 @@ class EditQuotationsDialog : public QDialog {
 		void addQuotation();
 		void changeQuotation();
 		void deleteQuotation();
+		void importQuotations();
+		void exportQuotations();
+		void onFilterSelected(QString filter);
 
 };
 
@@ -655,8 +712,10 @@ class RefundDialog : public QDialog {
 		Transactions *transaction;
 		EqonomizeValueEdit *valueEdit, *quantityEdit;
 		QDateEdit *dateEdit;
-		QComboBox *accountCombo;
+		AccountComboBox *accountCombo;
 		QLineEdit *commentsEdit;
+		
+		void keyPressEvent(QKeyEvent*);
 
 	public:
 
@@ -668,7 +727,7 @@ class RefundDialog : public QDialog {
 	protected slots:
 		
 		void accept();
-		void accountActivated(int);
+		void accountActivated(Account*);
 
 };
 
@@ -684,6 +743,8 @@ class EditSecurityTradeDialog : public QDialog {
 		EqonomizeValueEdit *fromSharesEdit, *toSharesEdit;
 		QDateEdit *dateEdit;
 		int prev_from_index;
+		
+		void keyPressEvent(QKeyEvent *e);
 
 	public:
 
@@ -702,6 +763,7 @@ class EditSecurityTradeDialog : public QDialog {
 		Security *selectedToSecurity();
 		void fromSecurityChanged(bool = false);
 		void toSecurityChanged();
+		void focusDate();
 
 };
 
@@ -765,5 +827,58 @@ class QIFWizardPage : public QWizardPage {
 		void setComplete(bool b);
 };
 
+class EqonomizeComboBox : public QComboBox {
+
+	Q_OBJECT
+	
+	protected:
+	
+		bool block_selected;
+
+	public:
+	
+		EqonomizeComboBox(QWidget *parent);
+	
+	protected slots:
+	
+		void keyPressEvent(QKeyEvent *event);
+		void itemActivated(int);
+		
+	signals:
+	
+		void returnPressed();
+		void itemSelected(int);
+
+};
+
+class EqonomizeItemDelegate : public QStyledItemDelegate {
+
+	Q_OBJECT
+
+	public:
+		EqonomizeItemDelegate(QObject *parent = NULL);
+
+	public slots:
+	
+		bool helpEvent(QHelpEvent *e, QAbstractItemView *view, const QStyleOptionViewItem &option, const QModelIndex &index);
+		void paint(QPainter *painter, const QStyleOptionViewItem& option, const QModelIndex &index) const;
+
+};
+
+
+class EqonomizeTranslator : public QTranslator {
+	
+	Q_OBJECT
+	
+	public:
+
+		EqonomizeTranslator();
+		
+		QString	translate(const char *context, const char *sourceText, const char *disambiguation = NULL, int n = -1) const;
+
+};
+
+void open_file_list(QString);
+
 #endif
 
