@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   Copyright (C) 2006-2008, 2014, 2016-2019 by Hanna Knutsson            *
  *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
  *   This file is part of Eqonomize!.                                      *
@@ -33,6 +33,7 @@
 #include <QMessageBox>
 #include <QDebug>
 #include <QFocusEvent>
+#include <QMenu>
 
 #define MAX_VALUE 1000000000000.0
 
@@ -64,16 +65,8 @@ void EqonomizeValueEdit::init(double lower, double upper, double step, double va
 	setSingleStep(step);
 	setAlignment(Qt::AlignRight);
 	if(show_currency) {
-		if(budget) {
-			if(i_precision < 0) i_precision = budget->defaultCurrency()->fractionalDigits(true);
-			setCurrency(budget->defaultCurrency(), true);
-		} else {
-			if(CURRENCY_IS_PREFIX) {
-				s_prefix = QLocale().currencySymbol();
-			} else {
-				s_suffix = QLocale().currencySymbol();
-			}
-		}
+		if(i_precision < 0) i_precision = budget->defaultCurrency()->fractionalDigits(true);
+		setCurrency(budget->defaultCurrency(), true);
 	}
 	if(i_precision < 0) i_precision = MONETARY_DECIMAL_PLACES;
 	setDecimals(i_precision);
@@ -81,6 +74,19 @@ void EqonomizeValueEdit::init(double lower, double upper, double step, double va
 	connect(this, SIGNAL(editingFinished()), this, SLOT(onEditingFinished()));
 	QLineEdit *w = findChild<QLineEdit*>();
 	if(w) connect(w, SIGNAL(textChanged(const QString&)), this, SLOT(onTextChanged(const QString&)));
+	w->installEventFilter(this);
+}
+bool EqonomizeValueEdit::eventFilter(QObject *, QEvent *e) {
+	if(e->type() == QMouseEvent::MouseButtonDblClick) {
+		QLineEdit *w = findChild<QLineEdit*>();
+		QString text = w->text();
+		int pos = w->cursorPosition();
+		if(pos >= text.length()) pos = text.length() - 1;
+		if(!text[pos].isNumber() && !text[pos].isSpace() && !text[pos].isPunct()) return false;
+		selectNumber();
+		return true;
+	}
+	return false;
 }
 void EqonomizeValueEdit::onTextChanged(const QString &text) {
 	if(text == s_suffix) {
@@ -119,10 +125,12 @@ void EqonomizeValueEdit::selectNumber() {
 		int end = text.length();
 		if(!s_prefix.isEmpty() && text.startsWith(s_prefix)) {
 			start = s_prefix.length();
+			if(text.length() > start && text[start] == ' ') start++;
+		}
+		if(!s_suffix.isEmpty() && text.endsWith(s_suffix)) {
+			end -= s_suffix.length();
+			if(end > 0 && text[end - 1] == ' ') end--;
 		}
-		if(!s_suffix.isEmpty() && text.endsWith(QString(" ") + s_suffix)) {
-			end -= (s_suffix.length() + 1);
-		}		
 		w->setSelection(end, start - end);
 	}
 }
@@ -151,8 +159,13 @@ void EqonomizeValueEdit::setCurrency(Currency *currency, bool keep_precision, in
 	}
 	
 	if((as_default == 0 || (as_default < 0 && currency != budget->defaultCurrency())) || currency->symbol().isEmpty()) {
-		s_suffix = currency->code();
-		s_prefix = QString();
+		if(currency && currency->codePrecedes()) {
+			s_prefix = currency->code();
+			s_suffix = QString();
+		} else {
+			s_suffix = currency->code();
+			s_prefix = QString();
+		}
 	} else {
 		if(currency && currency->symbolPrecedes()) s_prefix = currency->symbol();
 		else s_prefix = QString();
@@ -170,6 +183,13 @@ Currency *EqonomizeValueEdit::currency() {return o_currency;}
 
 QValidator::State EqonomizeValueEdit::validate(QString &input, int &pos) const {
 	QString input2 = input;
+	if(o_currency) {
+		input2.remove(budget->monetary_group_separator);
+		input2.replace(budget->monetary_decimal_separator, QLocale().decimalPoint());
+	} else {
+		input2.remove(budget->group_separator);
+		input2.replace(budget->decimal_separator, QLocale().decimalPoint());
+	}
 	int pos2 = pos;
 	QValidator::State s = QDoubleSpinBox::validate(input2, pos2);
 	if(s == QValidator::Invalid) {
@@ -177,8 +197,8 @@ QValidator::State EqonomizeValueEdit::validate(QString &input, int &pos) const {
 		if(!s_suffix.isEmpty() && str.endsWith(s_suffix)) {
 			str = str.left(str.length() - s_suffix.length());
 		}
-		if(!s_prefix.isEmpty() && str.endsWith(s_prefix)) {
-			str = str.right(str.length() - s_prefix.length());
+		if(!s_suffix.isEmpty()) {
+			str.remove(s_suffix);
 		}
 		str = str.simplified();
 		str.remove(' ');
@@ -192,8 +212,9 @@ QValidator::State EqonomizeValueEdit::validate(QString &input, int &pos) const {
 }
 
 QString EqonomizeValueEdit::textFromValue(double value) const {
-	if(!s_suffix.isEmpty()) return s_prefix + QLocale().toString(value, 'f', decimals()) + QString(" ") + s_suffix;
-	return s_prefix + QLocale().toString(value, 'f', decimals());
+	if(o_currency) return o_currency->formatValue(value, decimals(), true, false, true);
+	if(!s_suffix.isEmpty()) return s_prefix + budget->formatValue(value, decimals()) + QString(" ") + s_suffix;
+	return s_prefix + budget->formatValue(value, decimals());
 }
 double EqonomizeValueEdit::valueFromText(const QString &t) const {
 	QString str = t.simplified();
@@ -204,21 +225,37 @@ double EqonomizeValueEdit::valueFromText(const QString &t) const {
 		str.remove(s_prefix);
 	}
 	str.remove(' ');
+	if(o_currency) {
+		str.remove(budget->monetary_group_separator);
+		str.replace(budget->monetary_decimal_separator, QLocale().decimalPoint());
+	} else {
+		str.remove(budget->group_separator);
+		str.replace(budget->decimal_separator, QLocale().decimalPoint());
+	}
 	return QDoubleSpinBox::valueFromText(str);
 }
 
 void EqonomizeValueEdit::fixup(QString &input) const {
 	if(!s_prefix.isEmpty() && input.startsWith(s_prefix)) {
 		input = input.right(input.length() - s_prefix.length());
-		if(input.isEmpty()) input = QLocale().toString(1);
+		if(input.isEmpty()) input = QString::number(1);
 	}
 	if(!s_suffix.isEmpty() && input.endsWith(s_suffix)) {
 		input = input.left(input.length() - s_suffix.length());
-		if(input.isEmpty()) input = QLocale().toString(1);
+		if(input.isEmpty()) input = QString::number(1);
 	}
 	QString calculatedText_pre = input.trimmed();
+	input.replace("√", QString('^') + (o_currency ? o_currency->formatValue(0.5, -1, false, false, true) : budget->formatValue(0.5, 1)));
 	input.remove(QRegExp("\\s"));
-	input.remove(QLocale().groupSeparator());
+	if(o_currency) {
+		input.remove(budget->monetary_group_separator);
+		if(!budget->monetary_negative_sign.isEmpty()) input.replace(budget->monetary_negative_sign, "-");
+		if(!budget->monetary_positive_sign.isEmpty()) input.replace(budget->monetary_positive_sign, "+");
+	} else {
+		input.remove(budget->group_separator);
+		if(!budget->negative_sign.isEmpty()) input.replace(budget->negative_sign, "-");
+		if(!budget->positive_sign.isEmpty()) input.replace(budget->positive_sign, "+");
+	}
 	input.replace("⋅", "*");
 	input.replace("×", "*");
 	input.replace("−", "-");
@@ -228,10 +265,14 @@ void EqonomizeValueEdit::fixup(QString &input) const {
 	input.replace("³", "^3");
 	input.replace(']', ')');
 	input.replace('[', '(');
-	input.replace("√", QString('^') + QLocale().toString(0.5));
 	QStringList errors;
 	bool calculated = false;
-	input = QLocale().toString(fixup_sub(input, errors, calculated), 'f', decimals());
+	double v = fixup_sub(input, errors, calculated);
+	if(o_currency) {
+		input = o_currency->formatValue(v, decimals(), false, false, true);
+	} else {
+		input = budget->formatValue(v, decimals());
+	}
 	if(calculated && errors.isEmpty()) {
 		calculatedText = calculatedText_pre;
 		calculatedText_object = this;
@@ -258,6 +299,15 @@ double EqonomizeValueEdit::fixup_sub(QString &input, QStringList &errors, bool &
 	if(input.isEmpty()) {
 		return 0.0;
 	}
+	if(o_currency) {
+		input.remove(budget->monetary_group_separator);
+		if(!budget->monetary_negative_sign.isEmpty()) input.replace(budget->monetary_negative_sign, "-");
+		if(!budget->monetary_positive_sign.isEmpty()) input.replace(budget->monetary_positive_sign, "+");
+	} else {
+		input.remove(budget->group_separator);
+		if(!budget->negative_sign.isEmpty()) input.replace(budget->negative_sign, "-");
+		if(!budget->positive_sign.isEmpty()) input.replace(budget->positive_sign, "+");
+	}
 	input.replace(QLocale().negativeSign(), '-');
 	input.replace(QLocale().positiveSign(), '+');
 	int i = input.indexOf(')', 1);
@@ -292,7 +342,7 @@ double EqonomizeValueEdit::fixup_sub(QString &input, QStringList &errors, bool &
 		if(i < input.length() - 1 && (input[i + 1].isNumber() || input[i + 1] == '(')) input.insert(i + 1, '*');
 		QString str = input.mid(i2 + 1, i - i2 - 1);
 		double v = fixup_sub(str, errors, calculated);
-		input.replace(i2, i - i2 + 1, QLocale().toString(v, 'f', decimals() + 2));
+		input.replace(i2, i - i2 + 1, o_currency ? o_currency->formatValue(0.5, decimals() + 2, false, false, true) : budget->formatValue(v, decimals() + 2));
 		if(i2 > 0 && (input[i2 - 1].isNumber() || input[i2 - 1] == ')')) input.insert(i2, '*');
 		calculated = true;
 		return fixup_sub(input, errors, calculated);
@@ -387,12 +437,16 @@ double EqonomizeValueEdit::fixup_sub(QString &input, QStringList &errors, bool &
 		calculated = true;
 		return v;
 	}	
-	if(budget && o_currency) {
+	if(o_currency) {
 		QString reg_exp_str = "[\\d\\+\\-\\^";
 		reg_exp_str += '\\';
-		reg_exp_str += QLocale().decimalPoint();
+		reg_exp_str += budget->monetary_decimal_separator;
 		reg_exp_str += '\\';
-		reg_exp_str += QLocale().groupSeparator();
+		reg_exp_str += budget->monetary_group_separator;
+		if(budget->monetary_decimal_separator != "." && budget->monetary_group_separator != ".") {
+			reg_exp_str += '\\';
+			reg_exp_str += '.';
+		}
 		reg_exp_str += "]";
 		int i = input.indexOf(QRegExp(reg_exp_str));
 		if(i >= 1) {
@@ -463,20 +517,119 @@ double EqonomizeValueEdit::fixup_sub(QString &input, QStringList &errors, bool &
 		}
 	}
 	
-	if(!budget || !o_currency) {
+	if(!o_currency) {
 		QString reg_exp_str = "[^\\d\\+\\-";
 		reg_exp_str += '\\';
-		reg_exp_str += QLocale().decimalPoint();
+		reg_exp_str += budget->decimal_separator;
 		reg_exp_str += '\\';
-		reg_exp_str += QLocale().groupSeparator();
+		reg_exp_str += budget->group_separator;
+		if(budget->decimal_separator != "." && budget->group_separator != ".") {
+			reg_exp_str += '\\';
+			reg_exp_str += '.';
+		}
 		reg_exp_str += "]";
 		i = input.indexOf(QRegExp(reg_exp_str));
 		if(i >= 0) {
 			errors << tr("Unrecognized characters in expression.");
 		}
+		input.remove(budget->group_separator);
+		input.replace(budget->decimal_separator, ".");
+		return input.toDouble();
 	}
-	input.replace('-', QLocale().negativeSign());
-	input.replace('+', QLocale().positiveSign());
-	return QLocale().toDouble(input);
+	input.remove(budget->monetary_group_separator);
+	input.replace(budget->monetary_decimal_separator, ".");
+	return input.toDouble();
+}
+
+EqonomizeCalendarWidget::EqonomizeCalendarWidget(QWidget *parent) : QCalendarWidget(parent) {
+	setContextMenuPolicy(Qt::CustomContextMenu);
+	connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(popupContextMenu(const QPoint&)));
+	popupMenu = NULL;
+}
+void EqonomizeCalendarWidget::keyPressEvent(QKeyEvent *event) {
+	if(event->key() == Qt::Key_Home) {
+		setSelectedDate(QDate::currentDate());
+		event->accept();
+		return;
+	}
+}
+void EqonomizeCalendarWidget::popupContextMenu(const QPoint &pos) {
+	if(!popupMenu) {
+		popupMenu = new QMenu(this);
+		popupMenu->addAction(tr("Today"), this, SLOT(selectToday()));
+	}
+	popupMenu->popup(mapToGlobal(pos));
+}
+void EqonomizeCalendarWidget::selectToday() {
+	setSelectedDate(QDate::currentDate());
+}
+
+EqonomizeDateEdit::EqonomizeDateEdit(QWidget *parent) : QDateEdit(QDate::currentDate(), parent) {
+	setCalendarPopup(true);
+	setCalendarWidget(new EqonomizeCalendarWidget(this));
+	popupMenu = NULL;
+}
+EqonomizeDateEdit::EqonomizeDateEdit(const QDate &date, QWidget *parent) : QDateEdit(date, parent) {
+	setCalendarPopup(true);
+	setCalendarWidget(new EqonomizeCalendarWidget(this));
+	popupMenu = NULL;
+}
+void EqonomizeDateEdit::keyPressEvent(QKeyEvent *event) {
+	if(event->key() == Qt::Key_Down || event->key() == Qt::Key_Up) {
+		if(currentSection() == QDateTimeEdit::DaySection) {
+			QDate d = date();
+			if(event->key() == Qt::Key_Up && d.day() == d.daysInMonth()) {
+				setDate(d.addDays(1));
+				event->accept();
+				return;
+			} else if(event->key() == Qt::Key_Down && d.day() == 1) {
+				setDate(d.addDays(-1));
+				event->accept();
+				return;
+			}
+		} else if(currentSection() == QDateTimeEdit::MonthSection) {
+			QDate d = date();
+			if(event->key() == Qt::Key_Up && d.month() == 12) {
+				setDate(d.addMonths(1));
+				event->accept();
+				return;
+			} else if(event->key() == Qt::Key_Down && d.month() == 1) {
+				setDate(d.addMonths(-1));
+				event->accept();
+				return;
+			}
+		}
+	} else if((event->key() == Qt::Key_PageUp || event->key() == Qt::Key_PageDown)) {
+		if(currentSection() == QDateTimeEdit::DaySection) {
+			setDate(date().addMonths(event->key() == Qt::Key_PageUp ? 1 : -1));
+			event->accept();
+			return;
+		} else if(currentSection() == QDateTimeEdit::MonthSection) {
+			setDate(date().addYears(event->key() == Qt::Key_PageUp ? 1 : -1));
+			event->accept();
+			return;
+		}
+	} else if(event->key() == Qt::Key_Home && event->modifiers() == Qt::ControlModifier) {
+		setDate(QDate::currentDate());
+		event->accept();
+		return;
+	}
+	QDateEdit::keyPressEvent(event);
+	if(event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) {
+		event->accept();
+		emit returnPressed();
+	}
+}
+void EqonomizeDateEdit::setToday() {
+	setDate(QDate::currentDate());
+}
+void EqonomizeDateEdit::contextMenuEvent(QContextMenuEvent *event) {
+	if(!popupMenu) {
+		popupMenu = lineEdit()->createStandardContextMenu();
+		popupMenu->addSeparator();
+		todayAction = popupMenu->addAction(tr("Today"), this, SLOT(setToday()), Qt::CTRL | Qt::Key_Home);
+	}
+	todayAction->setEnabled(date() != QDate::currentDate());
+	popupMenu->exec(event->globalPos());
 }
 
