@@ -1,39 +1,42 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Hanna K.                                        *
- *   hanna_k@fmgirl.com                                                    *
+ *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
+ *   This file is part of Eqonomize!.                                      *
+ *                                                                         *
+ *   Eqonomize! is free software: you can redistribute it and/or modify    *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation, either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
+ *   Eqonomize! is distributed in the hope that it will be useful,         *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with Eqonomize!. If not, see <http://www.gnu.org/licenses/>.    *
  ***************************************************************************/
 
 #ifndef IMPORT_CSV_DIALOG_H
 #define IMPORT_CSV_DIALOG_H
 
 #include <QWizard>
+#include <QMap>
 
-class Budget;
 class QButtonGroup;
+class QCheckBox;
 class QLabel;
 class QRadioButton;
 class QSpinBox;
-class KDateEdit;
+class QComboBox;
+class QLineEdit;
+class QDateEdit;
+class QPushButton;
+
+class Budget;
 class EqonomizeValueEdit;
-class KLineEdit;
-class KComboBox;
-class KUrlRequester;
-class QCheckBox;
+
 struct csv_info;
 
 class ImportCSVDialog : public QWizard {
@@ -42,19 +45,24 @@ class ImportCSVDialog : public QWizard {
 	
 	protected:
 
+		bool b_extra;
 		Budget *budget;
+		
+		QMap<QString, QVariant> presets;
+		QString s_preset;
 
 		QLabel *typeDescriptionLabel;
-		QButtonGroup *typeGroup, *dateGroup, *valueGroup, *costGroup, *descriptionGroup, *AC1Group, *AC2Group, *commentsGroup;
+		QButtonGroup *typeGroup, *dateGroup, *valueGroup, *costGroup, *descriptionGroup, *AC1Group, *AC2Group, *commentsGroup, *payeeGroup, *quantityGroup, *tagsGroup;
 
-		KUrlRequester *fileEdit;
+		QLineEdit *fileEdit;
+		QPushButton *fileButton;
 		QSpinBox *rowEdit;
-		KComboBox *delimiterCombo;
-		KLineEdit *delimiterEdit;
+		QComboBox *delimiterCombo;
+		QLineEdit *delimiterEdit;
 
 		QRadioButton *columnDescriptionButton, *valueDescriptionButton;
 		QSpinBox *columnDescriptionEdit;
-		KLineEdit *valueDescriptionEdit;
+		QLineEdit *valueDescriptionEdit;
 
 		QLabel *valueLabel;
 		QRadioButton *columnValueButton, *valueValueButton;
@@ -68,21 +76,41 @@ class ImportCSVDialog : public QWizard {
 
 		QRadioButton *columnDateButton, *valueDateButton;
 		QSpinBox *columnDateEdit;
-		KDateEdit *valueDateEdit;
+		QDateEdit *valueDateEdit;
 
 		QLabel *AC1Label;
 		QRadioButton *columnAC1Button, *valueAC1Button;
 		QSpinBox *columnAC1Edit;
-		KComboBox *valueAC1Edit;
+		QComboBox *valueAC1Edit;
+		QComboBox *valueAC1IncomeEdit;
 
 		QLabel *AC2Label;
 		QRadioButton *columnAC2Button, *valueAC2Button;
 		QSpinBox *columnAC2Edit;
-		KComboBox *valueAC2Edit;
+		QComboBox *valueAC2Edit;
 
 		QRadioButton *columnCommentsButton, *valueCommentsButton;
 		QSpinBox *columnCommentsEdit;
-		KLineEdit *valueCommentsEdit;
+		QLineEdit *valueCommentsEdit;
+		
+		QLabel *tagsLabel;
+		QRadioButton *columnTagsButton, *valueTagsButton;
+		QSpinBox *columnTagsEdit;
+		QLineEdit *valueTagsEdit;
+		
+		QLabel *payeeLabel;
+		QRadioButton *columnPayeeButton, *valuePayeeButton;
+		QSpinBox *columnPayeeEdit;
+		QLineEdit *valuePayeeEdit;
+		
+		QLabel *quantityLabel;
+		QRadioButton *columnQuantityButton, *valueQuantityButton;
+		QSpinBox *columnQuantityEdit;
+		EqonomizeValueEdit *valueQuantityEdit;
+		
+		QComboBox *presetCombo;
+		QPushButton *savePresetButton;
+		QLabel *presetLabel;
 
 		QCheckBox *createMissingButton;
 
@@ -90,12 +118,15 @@ class ImportCSVDialog : public QWizard {
 		
 	public:
 		
-		ImportCSVDialog(Budget *budg, QWidget *parent);
+		ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *parent);
 		~ImportCSVDialog();		
 
 	protected slots:
 
+		void loadPreset(int);
+		void savePreset();
 		void onFileChanged(const QString&);
+		void selectFile();
 		void nextClicked();
 		void typeChanged(int);
 		void delimiterChanged(int);
