@@ -43,6 +43,7 @@ class QPushButton;
 class QRadioButton;
 
 class CategoryAccount;
+class AssetsAccount;
 class Budget;
 class EqonomizeMonthSelector;
 
@@ -60,7 +61,7 @@ class OverTimeChart : public QWidget {
 		Budget *budget;
 		bool b_extra;
 
-		QComboBox *sourceCombo, *categoryCombo, *descriptionCombo, *payeeCombo;
+		QComboBox *sourceCombo, *categoryCombo, *descriptionCombo, *payeeCombo, *accountCombo;
 		EqonomizeMonthSelector *startDateEdit, *endDateEdit;
 		QPushButton *saveButton, *printButton;
 		QRadioButton *yearlyButton, *dailyButton, *valueButton, *countButton, *perButton;
@@ -78,10 +79,10 @@ class OverTimeChart : public QWidget {
 #endif		
 		QButtonGroup *valueGroup;
 		
-		QDate start_date, end_date;
+		QDate start_date, end_date, saved_first_date, saved_last_date;
 
 		CategoryAccount *current_account;
-		QString current_description, current_payee;
+		QString current_description, current_payee, current_tag;
 		int current_source;
 		bool has_empty_description, has_empty_payee;
 #ifndef QT_CHARTS_LIB
@@ -93,11 +94,13 @@ class OverTimeChart : public QWidget {
 		void resetOptions();
 		void resetDate();
 		void sourceChanged(int);
+		void accountChanged(int);
 		void categoryChanged(int);
 		void descriptionChanged(int);
 		void payeeChanged(int);
 		void updateTransactions();
 		void updateAccounts();
+		void updateTags();
 		void updateDisplay();
 		void onFilterSelected(QString);
 		void save();
@@ -108,6 +111,7 @@ class OverTimeChart : public QWidget {
 		void endMonthChanged(const QDate&);
 		void endYearChanged(const QDate&);
 		void valueTypeToggled(bool);
+		AssetsAccount *selectedAccount();
 #ifdef QT_CHARTS_LIB
 		void themeChanged(int);
 		void typeChanged(int);
