@@ -44,6 +44,7 @@
 #include <QMessageBox>
 
 #include "budget.h"
+#include "eqonomizevalueedit.h"
 #include "recurrence.h"
 #include "recurrenceeditwidget.h"
 
@@ -83,8 +84,7 @@ EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : QDialog(parent, 0
 	layout->addWidget(infoLabel, 2, 0, 1, -1);
 
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
@@ -210,15 +210,14 @@ EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : QDia
 	endDateButton = new QRadioButton(tr("End on"), this);
 	rangeButtonGroup->addButton(endDateButton);
 	rangeLayout->addWidget(endDateButton, 3, 0);
-	endDateEdit = new QDateEdit(startdate, this);
+	endDateEdit = new EqonomizeDateEdit(startdate, this);
 	endDateEdit->setCalendarPopup(true);
 	rangeLayout->addWidget(endDateEdit, 3, 1, 1, 2);
 
 	setRecurrence(NULL);
 	
 	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
 	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
 	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
 	box1->addWidget(buttonBox);
@@ -317,7 +316,7 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	QVBoxLayout *recurrenceLayout = new QVBoxLayout(this);
 	//recurrenceLayout->setContentsMargins(0, 0, 0, 0);
 
-	recurrenceButton = new QCheckBox(tr("Enable recurrance"), this);
+	recurrenceButton = new QCheckBox(tr("Enable recurrence"), this);
 	recurrenceLayout->addWidget(recurrenceButton);
 
 	ruleGroup = new QGroupBox(tr("Recurrence Rule"), this);
@@ -367,7 +366,7 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	weeklyRuleLayout->addLayout(weeklyDaysLayout);
 	int weekStart = QLocale().firstDayOfWeek();	
 	for(int i = 0; i < 7; ++i ) {
-		weeklyButtons[i] = new QCheckBox(QDate::shortDayName(i + 1, QDate::StandaloneFormat), weeklyRuleWidget);
+		weeklyButtons[i] = new QCheckBox(QLocale().standaloneDayName(i + 1, QLocale::ShortFormat), weeklyRuleWidget);
 	}
 	for(int i = weekStart - 1; i < 7; ++i ) {
 		weeklyDaysLayout->addWidget(weeklyButtons[i]);
@@ -564,16 +563,16 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 		int months2 = 12;
 		if(months2 > months) {
 			for(int i2 = months + 1; i2 <= months2; i2++) {
-				yearlyMonthCombo_week->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
-				yearlyMonthCombo->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
+				yearlyMonthCombo_week->addItem(QLocale().monthName(i2, QLocale::LongFormat));
+				yearlyMonthCombo->addItem(QLocale().monthName(i2, QLocale::LongFormat));
 			}
 			months = months2;
 		}
 		date = date.addYears(1);
 	}
 	for(int i = 1; i <= 7; i++) {
-		yearlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
-		monthlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
+		yearlyDayOfWeekCombo->addItem(QLocale().standaloneDayName(i));
+		monthlyDayOfWeekCombo->addItem(QLocale().standaloneDayName(i));
 	}
 
 	connect(typeCombo, SIGNAL(activated(int)), ruleStack, SLOT(setCurrentIndex(int)));
