@@ -1,21 +1,21 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Hanna K.                                        *
- *   hanna_k@fmgirl.com                                                    *
+ *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
+ *   This file is part of Eqonomize!.                                      *
+ *                                                                         *
+ *   Eqonomize! is free software: you can redistribute it and/or modify    *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation, either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
+ *   Eqonomize! is distributed in the hope that it will be useful,         *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with Eqonomize!. If not, see <http://www.gnu.org/licenses/>.    *
  ***************************************************************************/
 
 #ifndef SECURITY_H
@@ -27,11 +27,15 @@
 #include <QList>
 
 #include "transaction.h"
+#include "eqonomizelist.h"
 
 class AssetsAccount;
-class QDomElement;
+class QXmlStreamReader;
+class QXmlStreamWriter;
+class QXmlStreamAttributes;
 class Security;
 class Budget;
+class Currency;
 
 typedef enum {
 	SECURITY_TYPE_BOND,
@@ -40,57 +44,6 @@ typedef enum {
 	SECURITY_TYPE_OTHER
 } SecurityType;
 
-class ReinvestedDividend {
-	public:
-		ReinvestedDividend(const QDate &date_, double shares_) : date(date_), shares(shares_) {}
-		QDate date;
-		double shares;
-};
-
-template<class type> class EqonomizeList : public QList<type> {
-	protected:
-		bool b_auto_delete;
-		int i_index;
-	public:
-		EqonomizeList() : QList<type>(), b_auto_delete(false), i_index(0) {};
-		virtual ~EqonomizeList () {}
-		void setAutoDelete(bool b) {
-			b_auto_delete = b;
-		}
-		void clear() {
-			if(b_auto_delete) {
-				while(!QList<type>::isEmpty()) delete QList<type>::takeFirst();
-			} else {
-				QList<type>::clear();
-			}
-		}		
-		bool removeRef(type value) {
-			if(b_auto_delete) delete value;
-			return QList<type>::removeOne(value);
-		}		
-		type first() {
-			i_index = 0;
-			if(QList<type>::isEmpty()) return NULL;
-			return QList<type>::first();
-		}
-		type getFirst() {
-			return QList<type>::first();
-		}
-		type next() {
-			i_index++;
-			if(i_index >= QList<type>::count()) return NULL;
-			return QList<type>::at(i_index);
-		}
-		type current() {
-			if(i_index >= QList<type>::count()) return NULL;
-			return QList<type>::at(i_index);
-		}
-		type previous() {			
-			if(i_index == 0) return NULL;
-			i_index--;
-			return QList<type>::at(i_index);
-		}
-};
 static bool security_transaction_list_less_than(Transaction *t1, Transaction *t2) {
 	return t1->date() < t2->date();
 }
@@ -98,40 +51,24 @@ template<class type> class SecurityTransactionList : public EqonomizeList<type>
 	public:
 		SecurityTransactionList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), security_transaction_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), security_transaction_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::append(value);
-			sort();
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, security_transaction_list_less_than), value);
 		}
 };
 static bool scheduled_security_list_less_than(ScheduledTransaction *t1, ScheduledTransaction *t2) {
-	return t1->transaction()->date() < t2->transaction()->date();
+	return t1->date() < t2->date();
 }
 
 template<class type> class ScheduledSecurityTransactionList : public EqonomizeList<type> {
 	public:
 		ScheduledSecurityTransactionList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), scheduled_security_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), scheduled_security_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::append(value);
-			sort();
-		}
-};
-static bool reinvested_dividend_list_less_than(ReinvestedDividend *t1, ReinvestedDividend *t2) {
-	return t1->date < t2->date;
-}
-template<class type> class ReinvestedDividendList : public EqonomizeList<type> {
-	public:
-		ReinvestedDividendList() : EqonomizeList<type>() {};
-		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), reinvested_dividend_list_less_than);
-		}
-		void inSort(type value) {
-			QList<type>::append(value);
-			sort();
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, scheduled_security_list_less_than), value);
 		}
 };
 static bool traded_shares_list_less_than(SecurityTrade *t1, SecurityTrade *t2) {
@@ -141,11 +78,10 @@ template<class type> class TradedSharesList : public EqonomizeList<type> {
 	public:
 		TradedSharesList() : EqonomizeList<type>() {};
 		void sort() {
-			qSort(QList<type>::begin(), QList<type>::end(), traded_shares_list_less_than);
+			std::sort(QList<type>::begin(), QList<type>::end(), traded_shares_list_less_than);
 		}
 		void inSort(type value) {
-			QList<type>::append(value);
-			sort();
+			QList<type>::insert(std::lower_bound(QList<type>::begin(), QList<type>::end(), value, traded_shares_list_less_than), value);
 		}
 };
 
@@ -155,7 +91,8 @@ class Security {
 
 		Budget *o_budget;
 
-		int i_id;
+		qlonglong i_id;
+		int i_first_revision, i_last_revision;
 
 		AssetsAccount *o_account;
 
@@ -163,7 +100,7 @@ class Security {
 
 		double d_initial_shares;
 
-		int i_decimals;
+		int i_decimals, i_quotation_decimals;
 
 		QString s_name;
 		QString s_description;
@@ -172,12 +109,23 @@ class Security {
 
 	public:
 
-		Security(Budget *parent_budget, AssetsAccount *parent_account, SecurityType initial_type, double initial_shares = 0.0, int initial_decimals = 4, QString initial_name = QString::null, QString initial_description = QString::null);
-		Security(Budget *parent_budget, QDomElement *e, bool *valid);
+		Security(Budget *parent_budget, AssetsAccount *parent_account, SecurityType initial_type, double initial_shares = 0.0, int initial_decimals = -1, int initial_quotation_decimals = -1, QString initial_name = QString(), QString initial_description = QString());
+		Security(Budget *parent_budget, QXmlStreamReader *xml, bool *valid);
+		Security(Budget *parent_budget);
 		Security();
 		Security(const Security *security);
 		virtual ~Security();
+		
+		void set(const Security *security);
+		void setMergeQuotes(const Security *security);
+		void mergeQuotes(const Security *security, bool keep = true);
 
+		virtual void readAttributes(QXmlStreamAttributes *attr, bool *valid);
+		virtual bool readElement(QXmlStreamReader *xml, bool *valid);
+		virtual bool readElements(QXmlStreamReader *xml, bool *valid);
+		virtual void save(QXmlStreamWriter *xml);
+		virtual void writeAttributes(QXmlStreamAttributes *attr);
+		virtual void writeElements(QXmlStreamWriter *xml);
 		const QString &name() const;
 		void setName(QString new_name);
 		const QString &description() const;
@@ -188,29 +136,35 @@ class Security {
 		void setInitialShares(double initial_shares);
 		virtual SecurityType type() const;
 		void setType(SecurityType new_type);
-		int id() const;
-		void setId(int new_id);
+		qlonglong id() const;
+		void setId(qlonglong new_id);
+		int firstRevision() const;
+		void setFirstRevision(int new_rev);
+		int lastRevision() const;
+		void setLastRevision(int new_rev);
 		bool hasQuotation(const QDate &date) const;
 		void setQuotation(const QDate &date, double value, bool auto_added = false);
 		void removeQuotation(const QDate &date, bool auto_added = false);
 		void clearQuotations();
 		double getQuotation(const QDate &date, QDate *actual_date = NULL) const;
 		AssetsAccount *account() const;
-		void addReinvestedDividend(const QDate &date, double added_shares);
+		Currency *currency() const;
 		int decimals() const;
+		int quotationDecimals() const;
 		void setDecimals(int new_decimals);
+		void setQuotationDecimals(int new_decimals);
 		void setAccount(AssetsAccount *new_account);
-		virtual void save(QDomElement *e) const;
 
 		double shares();
 		double shares(const QDate &date, bool estimate = false, bool no_scheduled_shares = false);
 		double value();
-		double value(const QDate &date, bool estimate = false, bool no_scheduled_shares = false);
-		double cost();
-		double cost(const QDate &date, bool no_scheduled_shares = false);
-		double profit();
-		double profit(const QDate &date, bool estimate = false, bool no_scheduled_shares = false);
-		double profit(const QDate &date1, const QDate &date2, bool estimate = false, bool no_scheduled_shares = false);
+		// estime > 0: estimate future value; estimate < 0: estimate value between quotations
+		double value(const QDate &date, int estimate = 0, bool no_scheduled_shares = false);
+		double cost(Currency *cur = NULL);
+		double cost(const QDate &date, bool no_scheduled_shares = false, Currency *cur = NULL);
+		double profit(Currency *cur = NULL);
+		double profit(const QDate &date, bool estimate = false, bool no_scheduled_shares = false, Currency *cur = NULL);
+		double profit(const QDate &date1, const QDate &date2, bool estimate = false, bool no_scheduled_shares = false, Currency *cur = NULL);
 		double yearlyRate();
 		double yearlyRate(const QDate &date);
 		double yearlyRate(const QDate &date_from, const QDate &date_to);
@@ -219,11 +173,12 @@ class Security {
 		QMap<QDate, double> quotations;
 		QMap<QDate, bool> quotations_auto;
 		TradedSharesList<SecurityTrade*> tradedShares;
-		ReinvestedDividendList<ReinvestedDividend*> reinvestedDividends;
 		SecurityTransactionList<SecurityTransaction*> transactions;
 		SecurityTransactionList<Income*> dividends;
+		SecurityTransactionList<ReinvestedDividend*> reinvestedDividends;
 		ScheduledSecurityTransactionList<ScheduledTransaction*> scheduledTransactions;
 		ScheduledSecurityTransactionList<ScheduledTransaction*> scheduledDividends;
+		ScheduledSecurityTransactionList<ScheduledTransaction*> scheduledReinvestedDividends;
 
 };
 
