@@ -1,53 +1,56 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Hanna K.                                        *
- *   hanna_k@fmgirl.com                                                    *
+ *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
+ *   This file is part of Eqonomize!.                                      *
+ *                                                                         *
+ *   Eqonomize! is free software: you can redistribute it and/or modify    *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation, either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
+ *   Eqonomize! is distributed in the hope that it will be useful,         *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with Eqonomize!. If not, see <http://www.gnu.org/licenses/>.    *
  ***************************************************************************/
 
-
 #ifndef TRANSACTION_FILTER_WIDGET_H
 #define TRANSACTION_FILTER_WIDGET_H
 
-#include <qvector.h>
-#include <qwidget.h>
-#include <qdatetime.h>
+#include <QVector>
+#include <QWidget>
+#include <QDateTime>
 
+class QButtonGroup;
+class QCheckBox;
+class QLabel;
+class QPushButton;
+class QRadioButton;
+class QCheckBox;
+
+class QComboBox;
+class QDateEdit;
+class QLineEdit;
+
+class Account;
 class Budget;
 class Transaction;
-class Account;
-class KLineEdit;
-class KDateEdit;
-class KComboBox;
-class QPushButton;
-class QLabel;
-class QCheckBox;
-class QRadioButton;
+class Transactions;
 class EqonomizeValueEdit;
-class QButtonGroup;
 
 class TransactionFilterWidget : public QWidget {
 
 	Q_OBJECT
-	
+
 	public:
 
 		TransactionFilterWidget(bool extra_parameters, int transaction_type, Budget *budg, QWidget *parent = 0);
 		~TransactionFilterWidget();
-		bool filterTransaction(Transaction *trans, bool checkdate = true);
+		bool filterTransaction(Transactions *transs, bool checkdate = true);
 		void updateFromAccounts();
 		void updateToAccounts();
 		void updateAccounts();
@@ -60,23 +63,25 @@ class TransactionFilterWidget : public QWidget {
 		QDate startDate();
 		QDate endDate();
 		void currentDateChanged(const QDate &olddate, const QDate &newdate);
+		void focusFirst();
+		void updateTags();
+
+		void setFilter(QDate fromdate, QDate todate, double min = -1.0, double max = -1.0, Account *from_account = NULL, Account *to_account = NULL, QString description = QString(), QString payee = QString(), bool exclude = false, bool exact_match = false, bool exclude_subs = false);
 
-		void setFilter(QDate fromdate, QDate todate, double min = -1.0, double max = -1.0, Account *from_account = NULL, Account *to_account = NULL, QString description = QString::null, QString payee = QString::null, bool exclude = false);
-	
 	protected:
 
 		QDate firstDate();
 		int transtype;
 		Budget *budget;
 		bool b_extra;
-		QVector<Account*> froms, tos;
-		KComboBox *fromCombo, *toCombo;
+		QComboBox *fromCombo, *toCombo, *tagCombo;
 		QCheckBox *minButton, *maxButton, *dateFromButton;
 		EqonomizeValueEdit *minEdit, *maxEdit;
-		KDateEdit *dateFromEdit, *dateToEdit;
-		KLineEdit *descriptionEdit, *payeeEdit;
+		QDateEdit *dateFromEdit, *dateToEdit;
+		QLineEdit *descriptionEdit;
 		QDate from_date, to_date;
 		QRadioButton *includeButton, *excludeButton;
+		QCheckBox *exactMatchButton, *excludeSubsButton;
 		QPushButton *clearButton;
 		QButtonGroup *group;
 
@@ -86,13 +91,15 @@ class TransactionFilterWidget : public QWidget {
 		void fromChanged(const QDate&);
 		void clearFilter();
 		void checkEnableClear();
+		void onToActivated(int);
+		void onFromActivated(int);
 
 	signals:
 
 		void filter();
-		void toActivated(int);
-		void fromActivated(int);
-		
+		void toActivated(Account*);
+		void fromActivated(Account*);
+
 };
 
 #endif
